#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

try:    import win32api, win32con
except:    pass

try:    from win32pipe import popen
except: from os import popen

import os, sys, string, stat, shutil, tempfile, time, re, binascii, subprocess, types
import zipfile, tarfile
from datetime import datetime

WIN32 = sys.platform == "win32"
USE_WIN32 = False
kSeperator = os.sep

class DvPath(object):
    """
    Common path and directory operations
    
    ...to be refactored...
    """

    def __init__(self, path = None):
        """ (string path) """
        self._path = u""
        self._pFindList = None
        self._pFindDirList = None
        self._pFindPos = 0
        self._pSearchSubDirs = False
        if path:
            if type(path) == types.UnicodeType:
                self.SetStr(path)
            else:
                self.SetStr(str(path))

    def __str__(self):
        return self._path

    def SetStr(self, path):
        """(string path)"""
        self._path = path

    def GetStr(self):
        """return string """
        return self._path


    # set ----------------------------------------------------------------------------
    def SetDrive(self, drive):
        """(string drive) """
        aD, aP = os.path.splitdrive(self._path)
        self._path = drive + aP

    def SetPath(self, path):
        """(string path) """
        aP, aNE = os.path.split(self._path)
        self._path = path
        self.AppendSeperator()
        self._path += aNE

    def SetName(self, n):
        """(string name) """
        aP, aStr = os.path.split(self._path)
        aStr, aE = os.path.splitext(self._path)
        if len(aP) > 0:
            aP += os.sep
        self._path = aP + n + aE

    def SetExtension(self, ext):
        """(string ext) """
        if not ext or len(ext) == 0:
            return
        if not ext[0] == ".":
            ext = "." + ext
        aP, aE = os.path.splitext(self._path)
        self._path = aP + ext

    def SetNameExtension(self, nameExt):
        """(string nameExt) """
        aP, aStr = os.path.split(self._path)
        self._path = aP
        self.AppendSeperator()
        self._path += nameExt

    def SetModTime(self, time):
        """(string time) return bool"""
        return False

    def AppendDirectory(self, inDir):
        self.AppendSeperator()
        self.SetName(inDir)
        self.AppendSeperator()

    def Append(self, inPath):
        """(string inPath) """
        if inPath == "":
            return
        if inPath[0] == os.sep:
            inPath = inPath[1:]
        self.AppendSeperator()
        self._path += inPath

    def RemoveLastDirectory(self):
        name = self.GetNameExtension()
        self.SetNameExtension("")
        if self.HasSeperatorEnd():
            self._path = self._path[:-1]
        self.SetNameExtension("")
        self.AppendSeperator()
        self.SetNameExtension(name)


    # get ----------------------------------------------------------------------------
    def GetDrive(self):
        """return string """
        if not self.IsValid():
            return ""
        aD, aP = os.path.splitdrive(self._path)
        return aD

    def GetPath(self):
        """return string
        Path includes trailing seperator
        """
        if not self.IsValid():
            return ""
        aP, aNE = os.path.split(self._path)
        if aP == "":
            return ""
        if not aP[-1] == os.sep:
            aP += os.sep
        return aP

    def GetName(self):
        """return string """
        if not self.IsValid():
            return ""
        aP, aNE = os.path.split(self._path)
        aN, aE = os.path.splitext(aNE)
        return aN

    def GetExtension(self):
        """return string """
        if not self.IsValid():
            return ""
        aP, aE = os.path.splitext(self._path)
        if len(aE) and aE[0] == ".":
            aE = aE[1:]
        return aE

    def GetNameExtension(self):
        """return string """
        if not self.IsValid():
            return ""
        aP, aNE = os.path.split(self._path)
        return aNE

    def GetAccessTime(self):
        """return datetime"""
        if not self.IsValid():
            return ""
        try:
            return datetime.fromtimestamp(os.stat(self._path)[stat.ST_ATIME])
        except:
            return 0

    def GetModTime(self):
        """return datetime"""
        if not self.IsValid():
            return ""
        try:
            return datetime.fromtimestamp(os.stat(self._path)[stat.ST_MTIME])
        except:
            return 0

    def GetSize(self):
        """return long"""
        if not self.IsValid():
            return ""
        if self.IsDirectory():
            return 0
        try:
            return os.stat(self._path)[stat.ST_SIZE]
        except:
            return -1

    def GetLastDirectory(self):
        """return string
        name of last directory
        """
        if not self.IsValid():
            return ""
        aP, aNE = os.path.split(self._path)
        if aP[-1] == os.sep:
            aP = aP[:-1]
        aL = string.split(aP, os.sep)
        if len(aL) == 0:
            return ""
        return aL[-1]

    def HasExtension(self):
        """return bool"""
        if not self.IsValid():
            return False
        return self._path.find(".") != -1


    # state ----------------------------------------------------------------------------
    def IsFile(self):
        """return bool """
        return os.path.isfile(self._path)

    def IsDirectory(self):
        """return bool """
        return os.path.isdir(self._path)

    def Exists(self):
        """return bool """
        return self.IsFile() or self.IsDirectory()

    def IsValid(self):
        """return bool """
        return len(self._path) > 0

    def IsEmpty(self):
        """return bool """
        if os.path.isdir(self._path):
            temp = self._path
            self.SetName("*")
            result = self.FindFirst(inSearchSubDirs = False, inReturnDirectories = True)
            self._path = temp
            return not result
        return self.GetSize()


    # find loop ----------------------------------------------------------------------------

    def FindFiles(self, regex = False):
        """
        regex = filename in path is a regular expression
        return list
        """
        if WIN32 and USE_WIN32:
            return self._FindFilesWin()
        return self._FindFilesUnix(regex)


    def FindFirst(self, inSearchSubDirs = False, inReturnDirectories = False, regex = False):
        """
        regex = filename in path is a regular expression
        return bool
        """
        if WIN32 and USE_WIN32:
            return self._FindFirstWin(inSearchSubDirs, inReturnDirectories)
        return self._FindFirstUnix(inSearchSubDirs, inReturnDirectories, regex)


    def FindNext(self):
        """return bool """
        if WIN32 and USE_WIN32:
            return self._FindNextWin()
        return self._FindNextUnix()


    def GetSubDirList(self, inAbsolut = False):
        """(bool inAbsolut) return list
        Excludes files with attribute hidden from search
        """
        if WIN32 and USE_WIN32:
            return self._GetSubDirListWin(inAbsolut)
        return self._GetSubDirListUnix(inAbsolut)


    def FindClose(self):
        """return void"""
        #if self._pFindList:
        #    del(self._pFindList)
        self._pFindDirList = None
        self._pFindPos = 0


    # seperators ----------------------------------------------------------------------------

    def HasSeperatorEnd(self):
        """return bool """
        if len(self._path) == 0:
            return 0
        if not self._path[-1] == os.sep:
            return 0
        return 1

    def AppendSeperator(self):
        """return void"""
        if self._path!="" and not self.HasSeperatorEnd():
            self._path += os.sep

    def ConvertSeperators(self):
        """
        """
        if self._path.find("\\") != -1 and kSeperator != "\\":
            self._path = self._path.replace("\\", kSeperator)
        if self._path.find("/") != -1 and kSeperator != "/":
            self._path = self._path.replace("/", kSeperator)


    # operations ----------------------------------------------------------------------------

    def Move(self, newPath):
        """(string newPath) return bool
        Move file to new path
        """
        try:
            aOldPath = self._path
            self._path = newPath
            self.CreateDirectories()
            os.rename(aOldPath, newPath)
            return True
        except:
            self._path = aOldPath
            return False


    def Copy(self, copyPath):
        """(string copyPath) return bool
        Copy file to copyPath. Directories are created if necessary
        """
        try:
            aOldPath = self._path
            self._path = copyPath
            self.CreateDirectories()
            shutil.copy2(aOldPath, copyPath)
            return True
        except Exception, e:
            self._path = aOldPath
            self.err = str(e)
            return False


    def CopyExcp(self, copyPath):
        """(string copyPath) return bool
        Copy file to copyPath. Directories are created if necessary
        """
        aOldPath = self._path
        self._path = copyPath
        self.CreateDirectories()
        shutil.copy2(aOldPath, copyPath)
        return True


    def CopyList(self, fileList, srcRoot = "", destRoot = ""):
        """(list fileList, string srcRoot, string destRoot) return bool
        Copy a list of files from a src directory to a dest directory. Static Function.
        """
        try:
            if srcRoot[-1] != kSeperator:
                srcRoot += kSeperator
            if destRoot[-1] != kSeperator:
                destRoot += kSeperator
            os.makedirs(destRoot)

            for file in fileList:
                aF = destRoot + file
                aP, aNE = os.path.split(aF)
                os.makedirs(aP)
                shutil.copy2(srcRoot + file, aF)
            return True
        except:
            return False


    def CopySub(self, destRoot = ""):
        """
        copy all subdirectories
        """
        if destRoot[-1] != kSeperator:
            destRoot += kSeperator
        aRoot = self.GetPath()
        if not self.FindFirst(True):
            return True
        aCopy = True
        while aCopy:
            aSrcPath = self._path
            aDestPath = destRoot + aSrcPath[len(aRoot):]
            try:
                aP, aNE = os.path.split(aDestPath)
                os.makedirs(aP)
            except:
                pass
            shutil.copy2(aSrcPath, aDestPath)
            aCopy = self.FindNext()
        return True


    def CopySubStatus(self, destRoot = "", statusFunction = None):
        """
        copy all subdirectories
        statusFunction is called after each file with parameters: (succeed, path, dest, size, errorstring)
        """
        if destRoot[-1] != kSeperator:
            destRoot += kSeperator
        aRoot = self.GetPath()
        if not self.FindFirst(True):
            return True
        aCopy = True
        while aCopy:
            aSrcPath = self._path
            aDestPath = destRoot + aSrcPath[len(aRoot):]
            try:
                aP, aNE = os.path.split(aDestPath)
                if not os.path.isdir(aP):
                    os.makedirs(aP)
                shutil.copy2(aSrcPath, aDestPath)
                if statusFunction:
                    statusFunction(True, aSrcPath, aDestPath, self.GetSize(), "")
            except Exception, e:
                statusFunction(False, aSrcPath, aDestPath, 0, str(e))
            aCopy = self.FindNext()
        return True


    def Delete(self, inDeleteSubdirs = False):
        """return bool """
        if inDeleteSubdirs:
            aRoot = self.GetPath()
            if not self.IsDirectory():
                return False
            self.AppendSeperator()
            self.SetNameExtension("*.*")
            if not self.FindFirst(True):
                try:
                    os.rmdir(self._path)
                except:
                    pass
                return not self.IsDirectory()
            aDel = True
            while aDel:
                if os.path.isfile(self._path):
                    os.remove(self._path)
                aDel = self.FindNext()
            self._path = aRoot
            try:
                os.removedirs(self._path)
            except:
                pass
            return not self.IsDirectory()

        if self.IsDirectory():
            os.rmdir(self._path)
            return not self.IsDirectory()
        try:
            os.remove(self._path)
        except:
            pass
        return not self.IsFile()


    def DeleteEmptyDirectories(self):
        """return bool """
        # list directories
        level = 0
        dirs = [[self._path]]
        path = self._path
        dirs = self._SubList(self._path, dirs, level+1)
        self._path = path

        # delete
        dirs.reverse()
        cnt = 0
        for l in dirs:
            for path in l:
                try:
                    os.rmdir(path)
                    cnt += 1
                except:
                    pass
        return cnt


    def CreateDirectories(self):
        """return bool"""
        if self.Exists():
            return True
        try:
            aP, aNE = os.path.split(self._path)
            os.makedirs(aP)
            return True
        except:
            return False


    def CreateDirectoriesExcp(self):
        """return bool"""
        if self.Exists():
            return True
        aP, aNE = os.path.split(self._path)
        os.makedirs(aP)
        return True


    def Rename(self, inNewName):
        """return bool"""
        if not self.IsFile():
            return False
        os.renames(self._path, inNewName)
        return True


    def ReadData(self):
        """
        return data
        compare this file with inFile
        """
        f = open(self._path)
        d = f.read()
        f.close()
        return d


    # system ----------------------------------------------------------------------------

    def SetUniqueTempFileName(self):
        """
        () return string
        """
        if not WIN32:
            aDir = tempfile.gettempdir()
            if not aDir:
                return False
            self._path = aDir
            self.AppendSeperator()
            aName = "tmp_" + str(time.time())
            self.SetNameExtension(aName)
            return True

        self._path, x = win32api.GetTempFileName(win32api.GetTempPath(), "tmp_", 0)
        return True


    def SetUniqueTempDir(self, prefix = "tmp_"):
        """
        () return string
        """
        if not WIN32:
            aDir = tempfile.gettempdir()
            if not aDir:
                return False
            self._path = aDir
            self.AppendSeperator()
            aName = prefix + str(time.time())
            self.AppendDirectory(aName)
            return True

        self._path, x = win32api.GetTempFileName(win32api.GetTempPath(), prefix, 0)
        return True


    def SetTempDir(self):
        """
        () return string
        """
        if not WIN32:
            aDir = tempfile.gettempdir()
            if not aDir:
                return False
            self._path = aDir
            self.AppendSeperator()
            return True

        self._path = win32api.GetTempPath()
        return True


    def SetModulePath(self, module=None):
        """
        set the path to module or DvLib module path
        """
        if not module:
            import DvLib
            self._path = nive.utils.path.__file__
            self.SetNameExtension("")
        else:
            self._path = module.__file__
            self.SetNameExtension("")


    def Execute(self, cmd, returnStream = True):
        """
        execute the current path with popen and returns std out
        """
        s = popen(self._path + " " + cmd, "r")
        if returnStream:
            return s
        r = ""
        while 1:
            d = s.readline()
            if not d or d == ".\n" or d == ".\r":
                break
            r += d
        return r


    def AddToPathEnv(self):
        """
        adds the current path to the PATH enviroment
        """
        path=os.environ["PATH"]
        if path.find(self._path) != -1:
            return
        path += ":" + self._path
        os.environ["PATH"] = path


    # archives -------------------------------------------------------------------------------------

    def Extract(self, destDirectory):
        """
        extract archive (zip or tar) to destination directory
        """
        if self.GetExtension() in ("zip",):
            return self.Unzip(destDirectory)
        return self.Untar(destDirectory)


    def Unzip(self, destDirectory):
        """
        unzip to destination directory
        """
        zfobj = zipfile.ZipFile(self._path)
        for name in zfobj.namelist():
            while name.startswith(os.sep):
                name = name[1:]
            name = name.replace("../","")
            if not name.endswith('/'): 
                root, file = os.path.split(name)
                directory = os.path.normpath(os.path.join(destDirectory, root))
                if not os.path.isdir(directory):
                    os.makedirs(directory)
                outfile = open(os.path.join(destDirectory, name), "wb")
                outfile.write(zfobj.read(name))
                outfile.close()

        return True


    def Untar(self, destDirectory):
        """
        unzip to destination directory
        """
        tar = tarfile.open(self._path, "r:gz")
        tar.extractall(destDirectory)
        return True


    def Pack(self, add=[], rootDir=""):
        """
        create a zip or tar archive from file/directory list.
        to store files with relative directory set rootdir
        to base directory for files 
        format is choosen from extension
        """
        if self.GetExtension()=="zip":
            return self.Zip(add, rootDir)
        return self.Tar(add, rootDir)

    
    def Zip(self, add=[], rootDir=""):
        zfobj = zipfile.ZipFile(self._path, mode="w")
        if rootDir != "" and not rootDir.endswith(os.sep):
            rootDir += os.sep
        for path in add:
            if rootDir and not path.startswith(rootDir):
                path = rootDir + path
            if os.path.isfile(path):
                zfobj.write(path, path[len(rootDir):], zipfile.ZIP_DEFLATED)
            else:
                self._ZipPath(zfobj, path, rootDir)
        zfobj.close()
        return True


    def _ZipPath(self, zfobj, path, rootDir):
        temp = self._path
        self._path = path
        self.SetName("*")
        f = self.FindFirst(inSearchSubDirs = True, inReturnDirectories = False)
        if not f:
            self._path = temp
            return False
        while f:
            if os.path.isfile(self._path):
                zfobj.write(self._path, self._path[len(rootDir):], zipfile.ZIP_DEFLATED)
            f = self.FindNext()
        self._path = temp
        return True
        

    def Tar(self, add=[], rootDir=""):
        files = ""
        for f in add:
            files += "'%s' "%(f)
        cmd = """cvzf %(path)s %(files)s""" % {"path": self._path, "files": str(files)}
        s = popen("tar " + cmd, "r")
        r = ""
        while 1:
            d = s.readline()
            if not d or d == ".\n" or d == ".\r":
                break
            r += d
        return True


    # directory rotation ----------------------------------------------------------------------------

    def AppendDirectoryRotation(self, period):
        d = datetime.now()
        if period == "day":
            return True
        elif period == "week":
            name = d.strftime("%A")
        elif period == "month":
            name = d.strftime("%d")
        elif period == "year-month":
            name = d.strftime("%Y-%m")
        elif period == "timestamp":
            name = d.strftime("%Y-%m-%d_%H%M%S")
        self.AppendSeperator()
        self.AppendDirectory(name)
        return True


    # internal -------------------------------------------------------------------------------

    def _SubList(self, path, dirs, level):
        self._path = path
        l = self.GetSubDirList(True)
        if len(dirs) <= level:
            dirs.append(l)
        else:
            dirs[level] = dirs[level] + l
        for p in l:
            dirs = self._SubList(p, dirs, level+1)
        return dirs


    # windows specific -------------------------------------------------------------------------------

    def _FindFilesWin(self):
        """return list """
        return win32api.FindFiles(self._path)


    def _FindFirstWin(self, inSearchSubDirs = False, inReturnDirectories = False):
        """return bool """
        try:    self._pFindList = win32api.FindFiles(self._path)
        except: return False
        self._pFindPos = -1
        self._pFindDirList = []
        self.pFindFile = self.GetNameExtension()
        self._pSearchSubDirs = inSearchSubDirs
        self._pReturnDirs = inReturnDirectories
        sys.setrecursionlimit(100000)
        return self.FindNext()


    def _FindNextWin(self):
        """return bool """
        if self._pFindList == None:
            return False

        self._pFindPos += 1
        if len(self._pFindList) == 0 or self._pFindPos >= len(self._pFindList):
            if not self._pSearchSubDirs:
                self.FindClose()
                return False

            aSubDirList = self.GetSubDirList(True)                   # search subdirectories
            self._pFindDirList.extend(aSubDirList)
            if len(self._pFindDirList) == 0:
                self.FindClose()
                return False
            self._pFindDirList.sort()

            self._path = self._pFindDirList.pop(0)                   # get NEXT directory
            self.AppendSeperator()
            self.SetNameExtension(self.pFindFile)
            self._pFindList = win32api.FindFiles(self._path)         # start find in new directory
            self._pFindPos = -1
            return self.FindNext()

        if ((self._pFindList[self._pFindPos][0] & win32con.FILE_ATTRIBUTE_DIRECTORY) >> 4):
            if not self._pReturnDirs:                                 # skip directories
                return self.FindNext()
            if self._pFindList[self._pFindPos][8] in ("..", "."):    # parent reference
                return self.FindNext()

        self.SetNameExtension(self._pFindList[self._pFindPos][8])

        return True


    def _GetSubDirListWin(self, inAbsolut = False):
        """(bool inAbsolut) return list
        Excludes files with attribute hidden from search
        """
        aP = self.GetPath()
        aP += "*.*"
        aL = win32api.FindFiles(aP)
        aDirs = []
        for aFI in aL:
            if aFI[8] == "." or aFI[8] == "..":
                continue
            if ((aFI[0] & win32con.FILE_ATTRIBUTE_DIRECTORY) >> 4):
                if ((aFI[0] & win32con.FILE_ATTRIBUTE_HIDDEN) >> 1):        # exclude files with attribute hidden from search
                    continue
                if inAbsolut:
                    aDirs.append(self.GetPath() + aFI[8] + os.sep)
                else:
                    aDirs.append(aFI[8] + os.sep)
        return aDirs


    # unix specific -------------------------------------------------------------------------------

    def _FindFilesUnix(self, regex = False):
        """return list """
        p = self.GetPath()
        for root, dirs, files in os.walk(p):
            fl = []
            for f in files:
                if not self._PathMatch(self.GetNameExtension(), f, regex):
                    continue
                fl.append(f)
            return fl
        return []


    def _FindFirstUnix(self, inSearchSubDirs = False, inReturnDirectories = False, regex = False):
        """return bool """
        self._pFindPos = -1
        self.regex = regex
        self._pFindDirList = []
        self.pFindFile = self.GetNameExtension()
        self._pSearchSubDirs = inSearchSubDirs
        self._pReturnDirs = inReturnDirectories
        sys.setrecursionlimit(100000)
        self._pFindList = self._FindFilesUnix(regex)
        return self._FindNextUnix()


    def _FindNextUnix(self):
        """return bool """
        if self._pFindList == None:
            return False
        if self._pFindDirList == None:
            return False

        self._pFindPos += 1
        while len(self._pFindList) == 0 or self._pFindPos >= len(self._pFindList):
            if not self._pSearchSubDirs:
                self.FindClose()
                return False

            aSubDirList = self._GetSubDirListUnix(True)                   # search subdirectories
            self._pFindDirList.extend(aSubDirList)
            if len(self._pFindDirList) == 0:
                self.FindClose()
                return False
            self._pFindDirList.sort()

            self._path = self._pFindDirList.pop(0)                   # get NEXT directory
            #print self._path
            self.AppendSeperator()
            self.SetNameExtension(self.pFindFile)
            self._pFindList = self._FindFilesUnix(self.regex)        # start find in new directory
            self._pFindPos = -1
            #return self.FindNext()

        self.SetNameExtension(self._pFindList[self._pFindPos])

        return True


    def _GetSubDirListUnix(self, inAbsolut = False):
        """(bool inAbsolut) return list
        Excludes files with attribute hidden from search
        """
        aDirs = []
        p = self.GetPath()
        for root, dirs, files in os.walk(p):
            for d in dirs:
                if inAbsolut:
                    aDirs.append(p + d + os.sep)
                else:
                    aDirs.append(d + os.sep)
            return aDirs
        return []


    def _PathMatch(self, path1, path2, regex):
        if not regex and path1.find("*") == -1:
            return True
        if not regex:
            s1=path1.split("*")
            #if s1[-1] != path2[-len(s1[-1]):]:
            #    return False
            path1 = path1.replace("\\", "\\\\")
            path1 = path1.replace(".", "\.")
            path1 = path1.replace("*", ".*")
        p = re.compile(path1, re.I)
        r = p.match(path2)
        return r


# --------------------------------------------------------------------------------

class DvDirCleaner(object):
    """
    Clean up directories

    baseDirectories
    regex
    """

    def __init__(self, baseDirectories, regex = False):
        """
        """
        if type(baseDirectories) == type(""):
            baseDirectories = [baseDirectories]
        self.baseDirectories = baseDirectories
        self.regex = regex


    def DeleteNotReferenced(self, searchDirectories, skipSearchExtensions = ["jpg","gif","pdf","jpeg","exe","bmp"], delete = False, searchSubdirectories = True):
        """
        Cleans directories and deletes all files which are not referenced.
        Directories to be cleaned are passed as baseDirectories. References
        are searched for in searchDirectories.
        """
        if type(searchDirectories) == type(""):
            searchDirectories = [searchDirectories]
        self.searchDirectories = searchDirectories
        self.skipSearchExtensions = skipSearchExtensions

        # get files to search reference for
        files = {}
        for base in self.baseDirectories:
            path = DvPath(base)
            if not path.FindFirst(searchSubdirectories, self.regex):
                continue
            files[str(path)] = 0
            while path.FindNext():
                files[str(path)] = 0

        if files == {}:
            return [], []

        # loop search directories
        checked = []
        for search in self.searchDirectories:
            search = DvPath(search)
            if not search.FindFirst(searchSubdirectories, self.regex):
                continue
            if search.GetExtension().lower() not in self.skipSearchExtensions:
                files = self._ProcessFile(search, files)
                checked.append(str(search))
            while search.FindNext():
                if search.GetExtension().lower() not in self.skipSearchExtensions:
                    files = self._ProcessFile(search, files)
                checked.append(str(search))

        # list not referenced
        l = []
        for ref in files.keys():
            if files[ref] == 0:
                l.append(ref)
        files = l

        # delete not referenced
        if delete:
            self._DeleteFiles(files)

            # remove unused references in files in base directories
            # run again until all not referenced are deleted
            if len(files) != 0:
                f2, c2 = self.DeleteNotReferenced(searchDirectories,skipSearchExtensions=skipSearchExtensions,  searchSubdirectories=searchSubdirectories, delete=delete)
                for f in f2:
                    if not f in files:
                        files.append(f)

        return files, checked


    def DeleteNotListed(self, fileList, skipSearchExtensions = [], delete = False, searchSubdirectories = True):
        """
        Processes each file in directory/subdirectories and search for reference in
        files in same or different directories. Not referenced files can be deleted.
        """
        # get files not in List
        files = []
        for base in self.baseDirectories:
            path = DvPath(base)
            if not path.FindFirst(searchSubdirectories, self.regex):
                continue
            if not path.GetExtension() in skipSearchExtensions and not str(path) in fileList:
                files.append(str(path))
            while path.FindNext():
                if not path.GetExtension() in skipSearchExtensions and not str(path) in fileList:
                    files.append(str(path))

        if not delete:
            return files

        # delete not referenced
        self._DeleteFiles(files)

        return files


    def MoveFilesList(self, fileList, bakupDirectory):
        """
        move the files in the list to a bakup directory
        uses self.baseDirectories to get relative path of files
        returns list with errors
        """
        err = []
        for f in fileList:
            rel = ""
            for b in self.baseDirectories:
                if f.find(b) == -1:
                    continue
                rel = f.replace(b, "")
                break
            file = DvPath(f)
            bakup = DvPath(bakupDirectory)
            bakup.Append(rel)
            if not file.Move(str(bakup)):
                err.append(str(bakup))
        return err


    def DeleteFiles(self, subdirectories = True, bakupDirectory = None):
        """
        delete all files matching base directories name/pattern
        if bakupDirectory is not none: files are moved to the directory
        returns list of deleted files
        """
        files = []
        for base in self.baseDirectories:
            path = DvPath(base)
            #path.Append
            if not path.FindFirst(subdirectories, self.regex):
                continue
            files.append(str(path))
            while path.FindNext():
                files.append(str(path))
        for f in files:
            p = DvPath(f)
            try:
                p.Delete()
            except:
                pass
        return files


    def DeleteEmptyDirectories(self):
        """
        delete empty subdirectories in base directories
        """
        n = 0
        for base in self.baseDirectories:
            path = DvPath(base)
            n += path.DeleteEmptyDirectories()
        return n



    def _ProcessFile(self, file, references):
        # load file data
        name = file.GetNameExtension()
        import utils
        data = utils.LoadFromFile(str(file))
        # loop files
        for ref in references.keys():
            if ref == str(file):
                continue
            refPath = DvPath(ref)
            refName = refPath.GetNameExtension()
            if data.find(refName) != -1:
                references[ref] = references[ref] + 1
        return references


    def _DeleteFiles(self, files):
        for f in files:
            p = DvPath(f)
            try:
                p.Delete()
            except:
                pass




#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

import hashlib
import thread
import time

from nive.definitions import Interface, implements
from nive.definitions import ModuleConf, Conf
from nive.security import UserFound


"""
Session user cache
------------------
Caches simplified user objects and attaches them to request objects
as request.environ["authenticated_user"]

Setup:
- adds SessionUserCache object to usedb.app as userdb.usercache
- 
- listens to root *getuser* events
- listens to user *login*, *logout*, *commit*, *delete*

"""

class ISessionUser(Interface):
    """ """


class SessionUserCache(object):
    """
    User caching support. Caches users including data as attributes. 
    
    Options: ::

        expires = objs are reloaded or purged after this many seconds. 0 = never expires 

    """
    expires = 20*60 

    def __init__(self, expires=None):
        if expires != None:
            self.expires = expires

    def Add(self, obj, id):
        """
        """
        try:
            lock = thread.allocate_lock()
            lock.acquire(1)
            setattr(self, self._Cachename(id), (obj, time.time()))
            if lock.locked():
                lock.release()
        except Exception, e:
            if lock and lock.locked():
                lock.release()

    def Get(self, id):
        """
        """
        n = self._Cachename(id)
        try:
            lock = thread.allocate_lock()
            lock.acquire(1)
            if hasattr(self, n):
                o = getattr(self, n)
                if lock.locked():
                    lock.release()
                return o[0]
        except:
            if lock and lock.locked():
                lock.release()
        return None

    def GetAll(self):
        """
        """
        objs = []
        try:
            lock = thread.allocate_lock()
            lock.acquire(1)
            for v in self.__dict__.keys():
                if v[:5] == "__c__":
                    objs.append(getattr(self, v)[0])
            if lock.locked():
                lock.release()
        except:
            if lock and lock.locked():
                lock.release()
        return objs

    def Invalidate(self, id):
        """
        """
        try:
            lock = thread.allocate_lock()
            lock.acquire(1)
            try:
                delattr(self, self._Cachename(id))
            except:
                pass
            if lock.locked():
                lock.release()
        except:
            if lock and lock.locked():
                lock.release()

    def Purge(self):
        """
        """
        try:
            lock = thread.allocate_lock()
            lock.acquire(1)
            tt = time.time()
            for v in self.__dict__.keys():
                if v[:5] == "__c__" and getattr(self, v)[1]+self.expires < tt:
                    delattr(self, v)
            if lock.locked():
                lock.release()
        except:
            if lock and lock.locked():
                lock.release()

    def _Cachename(self, id):
        return "__c__" + str(hash(str(id)))



class RootListener(object):
    
    def Init(self):
        self.ListenEvent("getuser", self.LookupCache)
        self.ListenEvent("loaduser", self.AddToCache)
    
    def LookupCache(self, ident=None, activeOnly=None):
        user = self.app.usercache.Get(ident)
        if user:
            raise UserFound, UserFound(user)

    def AddToCache(self, user=None, lastlogin=None):
        sessionuser = self.SessionUserFactory(user.identity, user)
        sessionuser.lastlogin=lastlogin
        self.app.usercache.Add(sessionuser, user.identity)
        
    def SessionUserFactory(self, ident, user):
        fields = ("name", "email", "surname", "lastname", "groups", "notify", "lastlogin")
        data = Conf()
        for f in fields:
            data[f] = user.data.get(f)
        data.lock()
        fields = ("id", "title", "pool_state")
        meta = Conf()
        for f in fields:
            meta[f] = user.data.get(f)
        meta.lock()
        su = SessionUser(ident, user.id, data, meta)
        return su


        
class UserListener(object):

    def Init(self):
        self.ListenEvent("commit", self.InvalidateCache)
        self.ListenEvent("logout", self.InvalidateCache)
    
    def InvalidateCache(self):
        self.app.usercache.Invalidate(self.identity)



class SessionUser(object):
    """
    The session user is created on login by the _real_ database user and cached on app
    level. In subsequent requests the session user is loaded from cache and attached to 
    the request. 
    
    All functions are readonly. The Session User is not connected to the database or 
    application. 
    
    Lifecycle: Login adds the user to the cache. Logout removes the user from the cache.
    Updates of user values also removes the user from cache.
     
    Default data values: name, email, surname, lastname, groups
    """
    implements(ISessionUser)
    
    def __init__(self, ident, id, data, meta=None):
        self.id = id
        self.identity = ident
        self.data = data
        self.meta = meta
        self.lastlogin = data.get(u"lastlogin")
        self.currentlogin = time.time()
    
    def __str__(self):
        return str(self.identity)

    @property
    def groups(self):
        return self.data.groups
    
    def GetGroups(self, context=None):
        """
        Returns the users gloabal groups as tuple.
        Local assignments are not supported, `context` is currently unused.
        """
        return self.data.groups

    def InGroups(self, groups):
        """
        check if user has one of these groups
        """
        if isinstance(groups, basestring):
            return groups in self.data.groups
        for g in groups:
            if g in self.data.groups:
                return True
        return False
    
    def ReadableName(self):
        if self.data.surname or self.data.lastname: 
            return u" ".join([self.data.surname, self.data.lastname])
        return self.data.name
    
    
# session user module definition

def SetupRootAndUser(app, pyramidConfig):
    # get all roots and user and add Listeners
    def add(confs, extension):
        for c in confs:
            e = c.extensions
            if e == None:
                e = []
            elif extension in e:
                continue
            if isinstance(e, tuple):
                e = list(e)
            e.append(extension)
            c.unlock()
            c.extensions = tuple(e)
            c.lock()
    
    rootextension = "nive.components.extensions.sessionuser.RootListener"
    add(app.GetAllRootConfs(), rootextension)
    userextension = "nive.components.extensions.sessionuser.UserListener"
    add([app.GetObjectConf("user",skipRoot=True)], userextension)
    # add usercache to app
    app.usercache = SessionUserCache()



configuration = ModuleConf(
    id = "sessionuser",
    name = u"Session user cache",
    events = (Conf(event="startRegistration", callback=SetupRootAndUser),),

)




#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

"""
nive Form description
------------------------
This class manages cms integration, field configuration, actions and form processing. 
Rendering, validation and data extraction is not included and requires a separate form 
library (nive.components.reform).

nive Forms can be connected to existing object or tool configurations and reuse all
therein defined fields for forms. To setup a form like this use ``Setup(subset='action to handled')`` 
to load fields, actions, subsets automatically from the objects configuration. 

A form does represent a single action but a set of form fields and multiple actions. nive forms can 
be rendered as ``subsets`` with a reduced selection of fields and different actions. 

This is an example to be used in object view code: ::

    form = ObjectForm(view=self, 
                      loadFromType=self.context.configuration)
    form.Setup(subset="edit") 
    # process and render the form.
    result, data, action = form.Process()

*view* must be an instance of `nive.views.BaseView`.  
*data* will contain the rendered HTML whether the is loaded for the first time, validated ok or 
not. *result* will be *false* if the form input did not validate. The *ObjectForm* already includes 
all necessary functions to load data initially, create an object and store data for existing objects.
All you have to do is switch the subset from *edit* to *create*. ::

    form = ObjectForm(view=self,
                      loadFromType=self.context.configuration)
    form.Setup(subset="create")
    # process and render the form.
    result, data, action = form.Process()

The configuration is loaded from the object configuration itself so all field settings are 
dynamically included. In fact the example above works for *any* object type.

Form action callback methods use the following footage: ::

    def Method(self, action, **kw):
        ...
        return result, data

These callback methods are automatically looked up and executed in Process(). Use action.method to
link a method to a specific form action. Pass `user` as kw argument if the forms view attribute is 
None. If form.view is set view.User() is used to lookup the current user.

A custom HTMLForm example: ::

    form = HTMLForm(view=self)
    form.actions = [
        Conf(id="default",    method="StartForm",             name=u"Initialize",          hidden=True),
        Conf(id="create",     method="ReturnDataOnSuccess",   name=u"Create a new group",  hidden=False),
    ]
    form.fields = [
        FieldConf(id="id",   name=u"Group id",   datatype="string", size="20", required=True),
        FieldConf(id="name", name=u"Group name", datatype="string", size="40", required=True)
    ]
    form.Setup()
    data, html, action = form.Process()


Callback for dynamic list item loading: ::

    def loadGroups(fieldconf, object):
        return [{"id": id, "name":name}]


    FieldConf(..., listItems=loadGroups, ...)


In short forms are explained as follows:

- fields are defined by field definitions as specified in :py:class:`nive.definitions.FieldConf`.
- configured fields are stored as list in self.fields.
- for display order and grouping self.subsets is used. Each subset is a list of field ids.
- subsets can be any selection and reference fields by id
- actions are mapped as buttons for the html form and callable for the result
- fields can directly be loaded from type or set manually. if loadFromType is specified, fields are loaded from this type. 
- create and edit can be used as default behavior for objects. 
- custom messages can be defined for errors and success of forms

**Please note:** These form classes are for autogenerating forms. If you need more control over validation
or rendering use a form library directly. 


Control sets for Select and Radio fields
========================================
Conditional sets can be automatically shown and hidden by setting a list `controlset:True`
and extending each listitem with a fields list. :: 

      FieldConf(id="flist",   
                datatype="list",   
                size=100,  
                name="flist", 
                settings={"controlset":True},
                listItems=[Conf(id="item 1", name="Item 1", 
                                fields=[FieldConf(id="ftext",datatype="text",size=10,name="ftext")]),
                           Conf(id="item 2", name="Item 2", 
                                fields=[FieldConf(id="fnum",datatype="number",size=8,name="fnum")])
                           ]
                ),

Example
=======
Internally the form uses a structure like in the following manually defined form example ::

    fields  = [
      FieldConf(id="ftext",   datatype="text",   size=1000, name="ftext", fulltext=1),
      FieldConf(id="fnumber", datatype="number", size=8,    name="fnumber", required=1),
      FieldConf(id="fdate",   datatype="datetime", size=8,  name="fdate"),
      FieldConf(id="flist",   datatype="list",   size=100,  name="flist", 
                listItems=[{"id": "item 1", "name":"Item 1"},
                             {"id": "item 2", "name":"Item 2"}]),
      FieldConf(id="fmselect", datatype="mselection", size=50, name="fmselect"),
    ]
    
    actions = [
      Conf(id="default",    method="StartForm", name="Initialize", hidden=True),
      Conf(id="defaultEdit",method="LoadUser",  name="Initialize", hidden=True),
      Conf(id="create",     method="AddUser",   name="Create",     hidden=False),
      Conf(id="edit",       method="Update",    name="Edit",       hidden=False)
    ]
    
    subsets = {
      "create": {"fields":  ["ftext", "fnumber", "fdate"], 
                 "actions": ["create"],
                 "defaultAction": "default"},
      "create2":{"fields":  ["ftext", "fnumber", "fdate", "flist", "fmselect"], 
                 "actions": ["create"],
                 "defaultAction": "default"},
      "edit":   {"fields":  ["ftext"], 
                 "actions": ["edit"],
                 "defaultAction": "defaultEdit"},
    }
    
    form = Form(view=self)
    form.fields = fields
    form.actions = actions
    form.subsets = subsets
    form.Setup(subset="create")
    # validating data
    result,data,errors = form.Validate(data)
    
Requires: Events

--------------------------------------------------------------------------
"""

import copy, json
from types import StringType

from pyramid.url import static_url

from nive.definitions import Conf, FieldConf, ConfigurationError
from nive.definitions import ISort
from nive.events import Events

from nive.components import reform
from nive.components.reform.form import Form as ReForm
from nive.components.reform.schema import null, Invalid
from nive.components.reform.exception import ValidationFailure

from nive.components.reform.reformed import SchemaFactory
from nive.components.reform.reformed import zpt_renderer
from nive.components.reform.i18n import _


class ValidationError(Exception):
    """
    Used for validation failures
    """

""" 
0.9.7 changed nive.Form:
- renamed parameter `redirect_success` to `redirectSuccess` and pass as **kw argument to functions
- form action method footage has changed: `def Method(self, action, **kw)`

0.9.4 changed nive.Form:
- __init__ parameters if view is not None: app, request, context are extracted from view context
- added Setup(subset) function
- removed LoadConfiguration()
- ObjectForm.AddTypeField() removed. Use Setup(addTypeField=True).
- form conf changed: object.configuration.forms = {"subset": {"fields": [...], "actions": [...]}}
- changed password widgets
"""

class Form(Events, ReForm):
    """
    Base form class.
    """
    _schemaFactory = SchemaFactory
    default_renderer = zpt_renderer

    # form configuration values
    fields = None
    actions = None
    subsets = None
    subset = None
    loadFromType = None
    
    
    def __init__(self, view=None, loadFromType=None, context=None, request=None, app=None, **kw):
        """
        Initialize form context. If view is not None and context, request, app are automatically
        extracted from the view object. Form fields and actions are processed in Setup(). 
        
        Please note: `view` must be an instance of `nive.views.BaseView`  
        """
        # form context
        self.view = view
        self.context = context or view.context
        self.request = request or view.request
        self.app = app or self.context.app
        
        if loadFromType:
            self.loadFromType = loadFromType

        # optional form parameters
        self.title = u""
        self.description = u""
        self.formid = u""
        self.startEmpty = False
        self.method = u"POST"
        
        # reform setup
        self._c_form = None
        self._c_fields = None
        self._c_actions = None
        ReForm.__init__(self, **kw)
        
        self.Signal("init")
            
            
    def Setup(self, subset=None):
        """
        1) Load fields from object definition
        2) Loads subsets and actions from object form definition
        
        Event
        - loadFields() after fields have been loaded
        """
        self._c_form = None
        self._c_fields = None
        self._c_actions = None
        self.subset=subset

        subsets=self.subsets
        config = None

        # load form fields
        if self.loadFromType:
            typeOrConfiguration = self.loadFromType
            if isinstance(typeOrConfiguration, basestring):
                config = self.app.GetObjectConf(typeOrConfiguration)
                if not config:
                    raise ConfigurationError, "Type not found (%s)" % (str(typeOrConfiguration))
            else:
                config = typeOrConfiguration
            self.loadFromType = config
        
        # unconfigured form
        if not subset and not subsets and not config and not self.fields:
            raise ConfigurationError, "No form fields defined"
        # unknown subset
        if subset and (not subsets or not subset in subsets):
            raise ConfigurationError, "Unknown form subset"

        # field lookup
        #(1) subsets[subset]["fields"]
        #(x) subsets[subset]
        #(3) type["form"][subset]["fields"]
        #(x) type["form"][subset]
        #(5) fields
        #(6) config.fields
        #(7) *fields.controlset=True -> field.listItems[].fields
        temp = None
        if subsets and subset in subsets and "fields" in subsets[subset]:
            #(1)
            temp = subsets[subset]["fields"]
        elif config and "forms" in config and config.forms and subset in config.forms and "fields" in config.forms[subset]:
            #(3)
            temp = config.forms[subset]["fields"]
        elif self.fields:
            temp = self.fields
        elif config and self.app:
            temp = list(self.app.GetAllMetaFlds(ignoreSystem = True)) + list(config.data)
        if not temp:
            raise ConfigurationError, "No form fields defined"
        # lookup field configurations
        self._c_fields = []
        for f in temp:
            if isinstance(f, basestring):
                fld = None
                if self.fields:
                    for a in config.data:
                        if a.id == f:
                            fld = a
                            break
                elif config:
                    for a in config.data:
                        if a.id == f:
                            fld = a
                            break
                if not fld:
                    fld = self.app.GetMetaFld(f)
                if not fld:
                    raise ConfigurationError, "Form field lookup failed: " + f
                f = fld
            self._c_fields.append(f)
            # add controlset fields
            if f.settings.get("controlset"):
                items = f.listItems
                if not isinstance(items, (list, tuple)):
                    items = items(f, self.context)
                for item in items:
                    if not item.get("fields"):
                        continue
                    for controlf in item.fields:
                        self._c_fields.append(controlf)
                    
        
        # action lookup
        #(1) subsets[subset]["actions"]
        #(2) type["form"][subset]["actions"]
        #(3) actions 
        temp = None
        if subsets and subset in subsets:
            if isinstance(subsets[subset], dict) and "actions" in subsets[subset]:
                #(1)
                temp = subsets[subset]["actions"]
        elif config and "forms" in config and config.forms and subset in config.forms:
            if isinstance(config.forms[subset], dict) and "actions" in config.forms[subset]:
                #(3)
                temp = config.forms[subset]["actions"]
        elif self.actions:
            temp = self.actions
        if temp:
            # lookup action configurations
            self._c_actions = []
            for a in temp:
                if isinstance(a, basestring):
                    action = None
                    if self.actions:
                        for v in self.actions:
                            if v.id == a:
                                action = v
                                break
                    if not action:
                        raise ConfigurationError, "Form action lookup failed: " + a
                    a = action
                self._c_actions.append(a)
                
        self.Signal("loadFields")
        
        
        
    def _SetUpSchema(self, force=False):
        """
        set up form schema for configured configuration.
        """
        if self._c_form:
            return 
        fields = self.GetFields()
        actions = self.GetActions(True)
        self._schemaFactory(self, fields, actions, force)
        self._c_form = True
        return 


    # Data extraction and validation --------------------------------------------------------------------------------------------

    def Validate(self, data, removeNull=True):
        """
        Extracts fields from request or source data dictionary, converts and
        validates
        
        Event
        - validate(data) before validate is called
        - process(data)  after validate has succeeded, is not called if validate failed
        
        returns bool, dictionary, list (result,data,errors)
        """
        self._SetUpSchema()
        error = None
        try:
            self.Signal("validate", data=data)
            data = self.validate(data)
        except ValidationFailure, e:
            return False, e.cstruct, e
        if removeNull:
            data = dict(filter(lambda d: d[1] != null, data.items()))
        self.Signal("process", data=data)
        return True, data, None


    def ValidateSchema(self, data, removeNull=True):
        """
        Extracts fields from request or source data dictionary, converts and
        validates
        
        Event
        - validate(data) before validate is called
        
        returns bool, dictionary, list (result,data,errors)
        """
        self._SetUpSchema()
        self.Signal("validate", data=data)
        errors = None
        result = True
        try:
            data = self.schema.deserialize(data)
        except Invalid, e:
            errors = e.children
            result = False
        
        if removeNull:
            data = dict(filter(lambda d: d[1] != null, data.items()))
        return result, data, errors

        
    def Extract(self, data, removeNull=True, removeEmpty=False):
        """
        Extract fields from request or source data dictionary and convert
        data types without validation. 
        
        returns bool, dictionary (result, data)
        """
        self._SetUpSchema()
        result = True
        try:
            data = self.validate(data)
        except ValidationFailure, e:
            result = False
            data = e.cstruct
        except ValueError:
            return self.ExtractDeserialized(data, removeNull)

        if removeNull:
            data = dict(filter(lambda d: d[1] != null, data.items()))
        if removeEmpty:
            data = dict(filter(lambda d: d[1] not in (u"",[],()), data.items()))
        
        return result, data
    
        
    def ExtractSchema(self, data, removeNull=False, removeEmpty=False):
        """
        Extract fields from request or source data dictionary and convert
        data types without validation. 
        
        returns bool, dictionary (result, data)
        """
        self._SetUpSchema()
        errors = None
        result = True
        try:
            data = self.schema.deserialize(data)
        except Invalid, e:
            result = False

        if removeNull:
            data = dict(filter(lambda d: d[1] != null, data.items()))
        if removeEmpty:
            data = dict(filter(lambda d: d[1] not in (u"",[],()), data.items()))

        return result, data


    # Loading initital data --------------------------------------------------------------------------------------------

    def LoadObjData(self, obj=None):
        """
        Load data from existing object
        
        Event
        - loadDataObj(data, obj) after data has been looked up
        
        returns dict
        """
        if not obj:
            obj = self.context
        data = {}
        for f in self.GetFields():
            # data
            if f.datatype == "file":
                data[f.id] = obj.files[f.id]
            if f.datatype == "password":
                data[f.id] = u""
            else:
                if obj.data.has_key(f.id):
                    data[f.id] = obj.data[f.id]
                elif obj.meta.has_key(f.id):
                    data[f.id] = obj.meta[f.id]
        self.Signal("loadDataObj", data=data, obj=obj)
        return data


    def LoadDefaultData(self):
        """
        Load default data from configuration
        
        Event
        - loadDefault(data) after data has been looked up
        
        returns dict
        """
        data = dict(filter(lambda y: y[1]!=None, 
                           map(lambda x: (x["id"],x["default"]), self.GetFields())))
        self.Signal("loadDefaultForm", data=data)
        return data


    # Field definitions and configuration --------------------------------------------------------------------------------------------

    def GetFields(self, removeReadonly=True):
        """
        Returns the list of field definitions. Readonly fields are removed by default.
        
        returns list
        """
        if not self._c_fields:
            return ()
        elif not removeReadonly:
            return self._c_fields
        return filter(lambda field: not (removeReadonly and field.readonly), self._c_fields)


    def GetActions(self, removeHidden=False):
        """
        Returns the list of form actions. Hidden actions are removed by default.
        
        returns list
        """
        if not self._c_actions:
            return ()
        elif not removeHidden:
            return self._c_actions
        return filter(lambda a: not (removeHidden and a.get("hidden",False)), self._c_actions)

    
    def GetField(self, fieldid):
        """
        Get the field configuration by field id.
        
        returns configuration or None
        """
        for f in self._c_fields:
            if f.id == fieldid:
                return f
        return None


    # form values --------------------------------------------------------------------------------------------

    def GetFormValue(self, key, request=None, method=None):
        """
        Extract single value from request
        
        returns value
        """
        if not request:
            request = self.request
        if isinstance(request, dict):
            return request.get(key)
        if not method:
            method = self.method
        try:
            if method == "POST":
                value = request.POST.getall(key)
            elif method == "GET":
                value = request.GET.getall(key)
            else:
                value = request.POST.getall(key)
                if not value:
                    value = request.GET.getall(key)
            if type(value)==StringType:
                value = unicode(value, self.app.configuration.frontendCodepage)
        except (AttributeError,KeyError):
            if method == "POST":
                value = request.POST.get(key)
            elif method == "GET":
                value = request.GET.get(key)
            else:
                value = request.POST.get(key)
                if not value:
                    value = request.GET.get(key)
            if type(value)==StringType:
                value = unicode(value, self.app.configuration.frontendCodepage)
            return value
        if not len(value):
            return None
        elif len(value)==1:
            return value[0]
        return value


    def GetFormValues(self, request=None, method=None):
        """
        Extract all values from request
        
        returns sict
        """
        if not request:
            request = self.request
        if not method:
            method = self.method
        try:
            if method == "POST":
                values = request.POST.mixed()
            elif method == "GET":
                values = request.GET.mixed()
            else:
                values = request.GET.mixed()
                values.update(request.POST.mixed())
        except AttributeError:
            try:
                if method == "POST":
                    values = request.POST
                elif method == "GET":
                    values = request.GET
                else:
                    values = request.GET
                    values.update(request.POST)
            except AttributeError:
                values = request
        if not values:
            return {}
        # should be unicode ?
        #cp=self.app.configuration.frontendCodepage
        #for k in values.items():
        #    if type(k[1]) == StringType:
        #        values[k[0]] = unicode(values[k[0]], cp)
        return values


    def ConvertFileUpload(self, key, method="post"):
        """
        Convert a file upload to the internal File object
        """
        value = self.GetFormValue(key, self.request, method=method)
        # prepare file
        file = self.app.db.GetFileClass()()
        file.filename = value.get('filename','')
        file.file = value.get('file')
        file.filekey = ticket.filekey
        file.mime = value.get('mimetype')
        file.size = value.get('size')
        file.tempfile = True
        return file



class HTMLForm(Form):
    """
    Simple HTML form 

    """
    # html styling
    formid = u"upload"
    css_class = u"form form-horizontal"
    actionPostfix = u"$"
    anchor = u""
    
    # Form actions --------------------------------------------------------------------------------------------

    def Process(self, **kw):
        """
        Processes the request and calls the required actions. kw parameter ::
        
            defaultAction: default action if none found in request. Can also be configured in subset
            redirectSuccess: default=None. url to redirect on action success 
            renderSuccess: default=False. render the form on action success 
        
        If renderSuccess is set to False the form itself will not be rendered if 
        the action succeeds. Only the feedback message will be rendered.
         
        Action definitions must define a callback method to process the action. 
        The callback takes two parameters: ::
    
            method(action, **kw)

        returns bool, html, dict (result, data, action)
        """
        # bw 0.9.7
        if u"redirect_success" in kw:
            redirectSuccess = kw[u"redirect_success"]

        # find default action
        defaultAction = None
        if self.subset:
            defaultAction = self.subsets[self.subset].get("defaultAction")
        if not defaultAction:
            defaultAction = kw.get("defaultAction", "default")
        
        # find action
        action = None
        formValues = self.GetFormValues(self.request)
        actions = self.GetActions()
        for a in actions:
            if a["id"]+self.actionPostfix in formValues.keys():
                action = a
                break

        if not action and defaultAction:
            # lookup default action 
            if isinstance(defaultAction, basestring):
                for a in self.actions:
                    if a["id"]==defaultAction:
                        action = a
                        break
            else:
                action = defaultAction

        # no action -> raise exception
        if not action:
            raise ConfigurationError, "No action to process the form found"

        # call action
        if isinstance(action["method"], basestring):
            method = getattr(self, action["method"])
        else:
            method = action["method"]
        
        # lookup and merge keyword parameter for action call
        methodKws = action.get("options")
        if methodKws:
            callKws = methodKws.copy()
            callKws.update(kw)
        else:
            callKws = kw
        result, html = method(action, **callKws)
        return result, html, action


    def IsRequestAction(self, action):
        """
        Check request for the given action.
        
        returns bool
        """
        formValues = self.GetFormValues(self.request)
        return action+self.actionPostfix in formValues.keys()


    def RemoveActionsFromRequest(self):
        """
        Removes all actions from the request. If called before processing the form
        the default action will be used and the form never processed.
        """
        actions = self.GetActions()
        formValues = self.GetFormValues(self.request)
        for a in actions:
            if a["id"]+self.actionPostfix in formValues.keys():
                action = a
                try:
                    del self.request.POST[a["id"]+self.actionPostfix]
                except:
                    pass
                try:
                    del self.request.GET[a["id"]+self.actionPostfix]
                except:
                    pass
            

    def StartForm(self, action, **kw):
        """
        Default action. Use this function to initially render a form if:
        - startEmpty is True
        - defaultData is passed in kw
        - load form default data

        returns bool, html
        """
        if self.startEmpty:
            data = {}
        elif "defaultData" in kw:
            data = kw["defaultData"]
        else:
            data = self.LoadDefaultData()
        return True, self.Render(data)


    def StartRequestGET(self, action, **kw):
        """
        Default action. Initially loads data from request GET values.
        Loads default data for initial from display on object creation.
        
        returns bool, html
        """
        data = self.GetFormValues(self.request, method="GET")
        return True, self.Render(data)


    def ReturnDataOnSuccess(self, action, **kw):
        """
        Process request data and returns validated data as `result` and rendered
        form as `data`. If validation fails `result` is False. `redirectSuccess`
        is ignored.
        
        A custom success message can be passed as success_message as keyword.

        returns bool, html
        """
        msgs = []
        result,data,errors = self.Validate(self.request)
        if result:
            if kw.get("success_message"):
                msgs.append(kw.get("success_message"))
            # disabled default message. msgs.append(_(u"OK."))
            errors=None
            result = data
            #data = {}
        return result, self.Render(data, msgs=msgs, errors=errors)

    
    def Cancel(self, action, **kw):
        """
        Cancel form action
        
        returns bool, string
        """
        redirectSuccess = kw.get("redirectSuccess")
        if self.view and redirectSuccess:
            self.view.Redirect(redirectSuccess)
        return True, ""


    # Data extraction and validation --------------------------------------------------------------------------------------------

    def Validate(self, data, removeNull=True):
        """
        Extracts fields from request or source data dictionary, converts and
        validates. 
        
        Event
        - validate(data) before validate is called
        - process(data)  after validate has succeeded, is not called if validate failed
        
        returns bool, dict, list (result,data,errors)
        """
        if not isinstance(data, dict):
            data = self.GetFormValues(data)
        result,data,errors = Form.Validate(self, data, removeNull)
        return result,data,errors

    
    def Extract(self, data, removeNull=False, removeEmpty=False):
        """
        Extracts fields from request or source data dictionary and converts
        data types without validation and error checking. 
        
        returns bool, dict (result, data)
        """
        if not isinstance(data, dict):
            data = self.GetFormValues(data)
        self._SetUpSchema()
        result, data = Form.Extract(self, data, removeNull, removeEmpty)
        return result, data


    # Form view functions --------------------------------------------------------------------------------------------

    def Render(self, data, msgs=None, errors=None, messagesOnly=False):
        """
        renders the form with data, messages
        
        messagesOnly=True will skip form and error rendering on just render the messages as 
        html block.
        """
        if messagesOnly:
            return self._Msgs(msgs=msgs)

        self._SetUpSchema()
        if errors:
            html = self._Msgs(msgs=msgs)
            return html + errors.render()
            #return html + exception.ValidationFailure(self._form, data, errors).render()
        self.messages = msgs
        html = self.render(data)
        return html


    def RenderBody(self, data, msgs=None, errors=None):
        """
        renders the form without header and footer
        """
        self._SetUpSchema()
        html = self._Msgs(msgs=msgs)
        if errors:
            return html + errors.render()
        self.widget.template = "form_body"
        html += self.render(data)
        self.widget.template = "form"
        return html

    
    def HTMLHead(self, ignore=(u"jquery.js",u"jquery-ui.js")):
        """
        Get necessary includes (js and css) for html header.
        Jquery and Jquery-ui are included by default in cmsview editor pages. So by default these two
        will be ignored. 
        """
        self._SetUpSchema()
        resources = self.get_widget_resources()
        js_resources = resources.get('js')
        css_resources = resources.get('css')
        resources = resources.get('seq')
                
        # js and css
        req = self.request
        js_links = []
        css_links = []
        if js_resources:
            js_links = [static_url(r, req) for r in filter(lambda v: v not in ignore, js_resources)]
        if css_resources:
            css_links = [static_url(r, req) for r in filter(lambda v: v not in ignore, css_resources)]
        # seq
        if resources:
            js_links.extend([static_url(r[1], req) for r in filter(lambda v: v[0] not in ignore and v[1].endswith(u".js"), resources)])
            css_links.extend([static_url(r[1], req) for r in filter(lambda v: v[0] not in ignore and v[1].endswith(u".css"), resources)])
        js_tags = [u'<script src="%s" type="text/javascript"></script>' % link for link in js_links]
        css_tags = [u'<link href="%s" rel="stylesheet" type="text/css" media="all"/>' % link for link in css_links]
        return (u"\r\n").join(js_tags + css_tags)
    
        
    def _FinishFormProcessing(self, result, data, msgs, errors, **kw):
        """
        Handles the default form processing after the action has been executed
        based on passed keywords and result:
        
        Used kw arguments:
        - redirectSuccess
        - renderSuccess
        
        """
        redirectSuccess = kw.get("redirectSuccess")
        renderSuccess = kw.get("renderSuccess", True)
        redirectSuccess = self.view.ResolveUrl(redirectSuccess, result)

        if not result:
            return result, self.Render(data, msgs=msgs, errors=errors)
    
        if self.use_ajax:
            if redirectSuccess:
                # raises HTTPFound 
                return result, self.view.Relocate(redirectSuccess, messages=msgs, raiseException=True)
        
        elif redirectSuccess:
            # raises HTTPFound 
            return result, self.view.Redirect(redirectSuccess, messages=msgs)

        if not renderSuccess:        
            return result, self._Msgs(msgs=msgs)
        return result, self.Render(data, msgs=msgs, errors=errors)


    def _Msgs(self, **values):
        err = values.get("errors")!=None
        msgs = values.get("msgs")
        if not msgs:
            return u""
        h = []
        if isinstance(msgs, basestring):
            msgs = [msgs]
        for m in msgs:
            h.append(u"""<li>%s</li>""" % (m))
        css = u"alert"
        if err:
            css = u"alert alert-danger"
        return u"""<div class="%s"><ul>%s</ul></div>
        """ % (css, u"".join(h))




# Preconfigured forms -----------------------------------------------------------------

class ObjectForm(HTMLForm):
    """
    Contains actions for object creation and updates.
    
    Supports sort form parameter *pepos*.
    """
    actions = [
        Conf(id=u"default",    method="StartFormRequest",  name=u"Initialize", hidden=True,  css_class=u"",            html=u"", tag=u""),
        Conf(id=u"create",     method="CreateObj",  name=u"Create",     hidden=False, css_class=u"btn btn-primary",  html=u"", tag=u""),
        Conf(id=u"defaultEdit",method="StartObject",name=u"Initialize", hidden=True,  css_class=u"",            html=u"", tag=u""),
        Conf(id=u"edit",       method="UpdateObj",  name=u"Save",       hidden=False, css_class=u"btn btn-primary",  html=u"", tag=u""),
        Conf(id=u"cancel",     method="Cancel",     name=u"Cancel",     hidden=False, css_class=u"buttonCancel",html=u"", tag=u"")
    ]
    subsets = {
        "create": {"actions": [u"create"],  "defaultAction": "default"},
        "edit":   {"actions": [u"edit"],    "defaultAction": "defaultEdit"}
    }


    def Setup(self, subset=None, addTypeField=False): #, addPosField=True
        """
        Calls Form.Setup() with the addition to automatically add the pool_type field. 
        
        1) Load fields from object definition
        2) Loads subsets and actions from object form definition
        
        Event
        - loadFields() after fields have been loaded
        """
        Form.Setup(self, subset)
        if not addTypeField:
            return
        #add type field 
        #opt
        pos = 0
        for field in self._c_fields:
            if field.id == "pool_type":
                if field.readonly:
                    # replace readonly field
                    type_fld = FieldConf(id="pool_type",datatype="string",hidden=1)
                    del self._c_fields[pos]
                    self._c_fields.append(type_fld)
                return
            pos += 1
        type_fld = FieldConf(id="pool_type",datatype="string",hidden=1)
        self._c_fields.append(type_fld)
        # insert at position
        if ISort.providedBy(self.context):
            #if addPosField:
            pepos = self.GetFormValue(u"pepos", method=u"ALL")
            if pepos:
                pos_fld = FieldConf(id="pepos",datatype="string",hidden=1)
                self._c_fields.append(pos_fld)


    def StartForm(self, action, **kw):
        """
        Default action. Called if no action in request or self.actions.default set.
        Loads default data for initial from display on object creation.
        
        returns bool, html
        """
        if self.startEmpty:
            data = {}
        else:
            data = self.LoadDefaultData()
        if isinstance(self.loadFromType, basestring):
            data["pool_type"] = self.loadFromType
        else:
            data["pool_type"] = self.loadFromType.id
        # insert at position
        if ISort.providedBy(self.context):
            pepos = self.GetFormValue(u"pepos", method=u"ALL")
            if pepos:
                data["pepos"] = pepos
        return True, self.Render(data)


    def StartFormRequest(self, action, **kw):
        """
        Default action. Called if no action in request or self.actions.default set.
        Loads default data from request for initial from display on object creation.
        
        returns bool, html
        """
        if self.startEmpty:
            data = {}
        else:
            data = self.LoadDefaultData()
            r, d = self.ExtractSchema(self.GetFormValues(method=u"ALL"), removeNull=True, removeEmpty=True)
            data.update(d)
        if isinstance(self.loadFromType, basestring):
            data["pool_type"] = self.loadFromType
        else:
            data["pool_type"] = self.loadFromType.id
        # insert at position
        if ISort.providedBy(self.context):
            pepos = self.GetFormValue(u"pepos", method=u"ALL")
            if pepos:
                data["pepos"] = pepos
        return True, self.Render(data)


    def StartObject(self, action, **kw):
        """
        Initially load data from object. 
        context = obj
        
        returns bool, html
        """
        data = self.LoadObjData()
        return data!=None, self.Render(data)


    def UpdateObj(self, action, **kw):
        """
        Process request data and update object.
        
        `Process()` returns the form data as result if update succeeds.

        Event
        - success(obj) after data has been successfully committed

        returns form data or false, html or redirects
        """
        redirectSuccess = kw.get("redirectSuccess")
        msgs = []
        obj=self.context
        result,data,errors = self.Validate(self.request)
        if result:
            user = kw.get("user") or self.view.User()
            result = obj.Update(data, user)
            if result:
                #obj.Commit(user)
                msgs.append(_(u"OK. Data saved."))
                self.Signal("success", obj=obj)
                result = obj

        return self._FinishFormProcessing(result, data, msgs, errors, **kw)


    def CreateObj(self, action, **kw):
        """
        Process request data and create object as child for current context.
        
        Pass kw['pool_type'] to specify type to be created. If not passed pool_type
        will be extracted from data dictionary.

        `Process()` returns the new object as result if create succeeds.

        Event
        - success(obj) after data has been successfully committed

        returns new object or none, html or redirects
        """
        redirectSuccess = kw.get("redirectSuccess")
        msgs = []
        result,data,errors = self.Validate(self.request)
        if result:
            if "pool_type" in kw:
                objtype = kw["pool_type"]
            else:
                objtype = data.get("pool_type")
                
            user=kw.get("user") or self.view.User()
            result = self.context.Create(objtype, data, user)
            if result:
                if ISort.providedBy(self.context):
                    # insert at position
                    pepos = self.GetFormValue(u"pepos")
                    if pepos:
                        self.context.InsertAtPosition(result, pepos, user=user)
                msgs.append(_(u"OK. Data saved."))
                self.Signal("success", obj=result)

        return self._FinishFormProcessing(result, data, msgs, errors, **kw)


class ToolForm(HTMLForm):
    """
    Contains default actions for tool form processing.

    Example ::

        tool = app.GetTool("tool id", contextObject=app)
        form = ToolForm(view=self, context=tool, loadFromType=tool.configuration)
        form.Setup()
        result, data, action = form.Process()

    A simple example to render a tool form and execute it.
    """
    actions = [
        Conf(id=u"default", method="StartForm", name=u"Initialize", hidden=True,  css_class=u"", html=u"", tag=u""),
        Conf(id=u"run",     method="RunTool",   name=u"Start",      hidden=False, css_class=u"", html=u"", tag=u""),
    ]

    def Setup(self, subset=None):
        """
        1) Load fields from tool definition
        2) Loads subsets and actions from tool form definition
        
        Event
        - loadFields() after fields have been loaded
        """
        self._c_form = None
        self._c_fields = None
        self._c_actions = None
        self.subset=subset

        subsets=self.subsets
        config = None

        # load form fields
        if self.loadFromType:
            typeOrConfiguration = self.loadFromType
            if isinstance(typeOrConfiguration, basestring):
                config = self.app.GetToolConf(typeOrConfiguration)
                if not config:
                    raise ConfigurationError, "Tool not found (%s). Use configuration instance instead of a string." % (str(typeOrConfiguration))
            else:
                config = typeOrConfiguration
            self.loadFromType = config
        
        # unconfigured form
        if not subset and not subsets and not config and not self.fields:
            raise ConfigurationError, "No form fields defined."
        # unknown subset
        if subset and (not subsets or not subset in subsets):
            raise ConfigurationError, "Unknown subset."

        # field lookup
        #(1) subsets[subset]["fields"]
        #(3) tool["form"][subset]["fields"]
        #(5) fields
        #(6) config.fields
        temp = None
        if subsets and subset in subsets and "fields" in subsets[subset]:
            #(1)
            temp = subsets[subset]["fields"]
        elif config and "forms" in config and config.forms and subset in config.forms and "fields" in config.forms[subset]:
            #(3)
            temp = config.forms[subset]["fields"]
        elif self.fields:
            temp = self.fields
        elif config and self.app:
            temp = config.data
        if not temp:
            raise ConfigurationError, "No form fields defined."
        # lookup field configurations
        self._c_fields = []
        for f in temp:
            if isinstance(f, basestring):
                fld = None
                if self.fields:
                    for a in config.data:
                        if a.id == f:
                            fld = a
                            break
                elif config:
                    for a in config.data:
                        if a.id == f:
                            fld = a
                            break
                if not fld:
                    raise ConfigurationError, "Form field lookup failed: " + f
                f = fld
            self._c_fields.append(f)
        
        # action lookup
        #(1) subsets[subset]["actions"]
        #(2) tool["form"][subset]["actions"]
        #(3) actions 
        temp = None
        if subsets and subset in subsets:
            if isinstance(subsets[subset], dict) and "actions" in subsets[subset]:
                #(1)
                temp = subsets[subset]["actions"]
        elif config and "forms" in config and config.forms and subset in config.forms:
            if isinstance(config.forms[subset], dict) and "actions" in config.forms[subset]:
                #(3)
                temp = config.forms[subset]["actions"]
        elif self.actions:
            temp = self.actions
        if temp:
            # lookup action configurations
            self._c_actions = []
            for a in temp:
                if isinstance(a, basestring):
                    action = None
                    if self.actions:
                        for v in self.actions:
                            if v.id == a:
                                action = v
                                break
                    if not action:
                        raise ConfigurationError, "Form action lookup failed: " + a
                    a = action
                self._c_actions.append(a)
                
        self.Signal("loadFields")
        
            
    def RunTool(self, action, **kw):
        """
        Process request data and run tool. 

        returns bool, html
        """
        msgs = []
        validated,data,errors = self.Validate(self.request)
        if validated:
            return self.context.Run(request=self.request,**data)
        return False, self.Render(data, msgs=msgs, errors=errors)




class WorkflowForm(HTMLForm):
    """
    Contains default actions for workflow transition form processing
    Requires Form, HTMLForm or TemplateForm
    """

    def CallWf(self, action, **kw):
        """
        process request data and call workflow transition for object. 
        context = obj
        """
        redirectSuccess = kw.get("redirectSuccess")
        msgs = []
        result,data,errors = self.Validate(self.request)
        if result:
            wfa = ""
            wft = ""
            user = kw.get("user") or self.view.User()
            if not obj.WfAction(action=wfa, transition=wft, user = user):
                result = False

        return self._FinishFormProcessing(result, data, msgs, errors, **kw)
        


class JsonMappingForm(HTMLForm):
    """
    Maps form fields to a single dumped json text field
    json data is always stored as dictionary. ::
    
        jsonDataField = the field to merge data to
        mergeContext = if true the database values will be updated
                       by form values
                       
    `Process()` returns the form data as dictionary on success.
    
    """
    jsonDataField = "data"
    mergeContext = True

    actions = [
        Conf(id=u"default",    method="StartObject",name=u"Initialize", hidden=True,  css_class=u"",                 html=u"", tag=u""),
        Conf(id=u"edit",       method="UpdateObj",  name=u"Save",       hidden=False, css_class=u"btn btn-primary",  html=u"", tag=u""),
    ]

    def Validate(self, data, removeNull=True):
        """
        Extracts fields from request or source data dictionary, converts and
        validates. 
        
        Event
        - validate(data) before validate is called
        - process(data)  after validate has succeeded, is not called if validate failed
        
        returns bool, dict, list (result,data,errors)
        """
        if not isinstance(data, dict):
            data = self.GetFormValues(data)
        result,data,errors = Form.Validate(self, data, removeNull)
        if not result:
            return result,data,errors
        # merge existing data
        if self.mergeContext and self.context:
            jdata = self.context.data[self.jsonDataField] or {}
            jdata.update(data)
            data = jdata
        return result,data,errors

    
    def Extract(self, data, removeNull=False, removeEmpty=False):
        """
        Extracts fields from request or source data dictionary and converts
        data types without validation and error checking. 
        
        returns bool, dict (result, data)
        """
        if not isinstance(data, dict):
            data = self.GetFormValues(data)
        self._SetUpSchema()
        result, data = Form.Extract(self, data, removeNull, removeEmpty)
        # merge existing data
        if self.mergeContext and self.context:
            jdata = self.context.data[self.jsonDataField]
            jdata.update(data)
            data = jdata
        return result,data


    def StartObject(self, action, **kw):
        """
        Initially load data from configured object json data field. 
        context = obj
        
        returns bool, html
        """
        data = self.context.data.get(self.jsonDataField) or {}
        return data!=None, self.Render(data)


    def UpdateObj(self, action, **kw):
        """
        Process request data and update object.
        
        Event
        - success(data) after data has been successfully committed

        returns bool, html
        """
        redirectSuccess = kw.get("redirectSuccess")
        msgs = []
        obj=self.context
        result,data,errors = self.Validate(self.request)
        if result:
            user = kw.get("user") or self.view.User()
            result = obj.Update({self.jsonDataField: data}, user)
            if result:
                #obj.Commit(user)
                msgs.append(_(u"OK. Data saved."))
                result = data
                self.Signal("success", data=data)

        return self._FinishFormProcessing(result, data, msgs, errors, **kw)
        
        

class JsonSequenceForm(HTMLForm):
    """
    Maps form fields as sequence to a single dumped json text field
    the sequence is always stored as list, json data as dictionary. 
    
    The form fields/values can be stored multiple times as sequence and
    the template offers options to add, delete and edit single items.
    Items in read only mode are not rendered by the form. 
    
    Sequence items are referenced by list position starting by 1.
    ::
    
        jsonDataField = the field to merge data to
                       
    `Process()` returns the form data as dictionary on success.
    
    """
    jsonDataField = u"data"
    jsonUniqueID = u"name"
    delKey = u"aa876352"
    editKey = u"cc397785"
    
    actions = [
        Conf(id=u"default",    method="StartObject",name=u"Initialize", hidden=True,  css_class=u"",                 html=u"", tag=u""),
        Conf(id=u"edit",       method="UpdateObj",  name=u"Save",       hidden=False, css_class=u"btn btn-primary",  html=u"", tag=u""),
    ]
    editKeyFld = FieldConf(id=editKey, name=u"indexKey", datatype="number", hidden=True, default=u"")
    
    def Init(self):
        self.ListenEvent("loadFields", "AddKeyFld")
        
    def AddKeyFld(self):
        self._c_fields.append(self.editKeyFld)


    def StartObject(self, action, **kw):
        """
        Initially load data from configured object json data field. 
        context = obj
        
        returns bool, html
        """
        sequence = self.context.data.get(self.jsonDataField)
        if sequence in (u"", None):
            sequence = []
        # process action
        msgs = []
        if self.GetFormValue(self.editKey, self.request, "GET"):
            seqindex = int(self.GetFormValue(self.editKey, self.request, "GET"))
            if not seqindex or seqindex > len(sequence):
                data = []
            else:
                data = sequence[seqindex-1]
                data[self.editKey] = seqindex
        elif self.GetFormValue(self.delKey, self.request, "GET"):
            seqindex = int(self.GetFormValue(self.delKey, self.request, "GET"))
            if not seqindex or seqindex > len(sequence):
                data = []
            else:
                del sequence[seqindex-1]
                self.context.data[self.jsonDataField] = sequence
                self.context.Commit(kw.get("user") or self.view.User())
                self.context.data[self.jsonDataField] = sequence
                msgs=[_(u"Item deleted!")]
                data = []
        else:
            data = []
        return data!=None, self.Render(data, msgs=msgs)


    def UpdateObj(self, action, **kw):
        """
        Process request data and update object.
        
        Event
        - success(data) after data has been successfully committed

        returns bool, html
        """
        redirectSuccess = kw.get("redirectSuccess")
        msgs = []
        result,data,errors = self.Validate(self.request)
        if result:
            user = kw.get("user") or self.view.User()
            # merge sequence list with edited item
            sequence = self.context.data.get(self.jsonDataField)
            if sequence in (u"", None):
                sequence = []
            try:
                seqindex = int(data.get(self.editKey))
            except:
                seqindex = 0
            if self.editKey in data:
                del data[self.editKey]
            if not seqindex or seqindex > len(sequence):
                append = 1
                # replace existing key
                if self.jsonUniqueID:
                    for item in sequence:
                        if item[self.jsonUniqueID] == data[self.jsonUniqueID]:
                            item.update(data)
                            append=0
                            break
                if append:
                    sequence.append(data)
            else:
                sequence[seqindex-1] = data
            self.context.data[self.jsonDataField] = sequence
            self.context.Commit(user)
            self.context.data[self.jsonDataField] = sequence
            msgs.append(_(u"OK. Data saved."))
            self.Signal("success", data=sequence)
            data = {}

        return result, self.Render(data, msgs=msgs, errors=errors)
        
        


# -*- coding: utf-8 -*-
#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
This file contains all base classes for subclassing your own objects. All required components
are already included as parent classes.
"""

from nive.application import Application, AppFactory, Configuration, Registration
from nive.container import Root, Container, ContainerEdit, ContainerSecurity, ContainerFactory, RootWorkflow
from nive.objects import Object, ObjectEdit, ObjectWorkflow
from nive.events import Events
from nive.search import Search
from nive.definitions import implements
from nive.definitions import IApplication, IContainer, IRoot, IReadonly, INonContainer, IObject
from nive.definitions import IPage, IPageContainer, IPageElement, IFile, IPageElementContainer, IFolder


class ApplicationBase(Application, AppFactory, Configuration, Registration, Events):
    """
    *Nive cms application* 
    
    The application manages module registration, module configuration, root dispatching
    and basic application events.
    """
    implements(IApplication)


class RootBase(Root, Container, Search, ContainerEdit, ContainerSecurity, Events, ContainerFactory, RootWorkflow):
    """
    *Root Edit*
    
    Default root class with add and delete support for subobjects. 
    """
    implements(IContainer, IRoot)
    
class RootReadOnlyBase(Root, Container, Search, Events, ContainerFactory, RootWorkflow):
    """
    *Root with readonly access and cache*
    
    Root class without add and delete support for subobjects. Objects are cached in memory.
    """
    implements(IRoot, IContainer, IReadonly)

    
    
class ObjectBase(Object, ObjectEdit, Events, ObjectWorkflow):
    """
    *Default non-container object with write access*
    
    This one does not support subobjects. 
    """
    implements(INonContainer, IObject)
    
class ObjectReadOnlyBase(Object, Events, ObjectWorkflow):
    """
    *Non-container object with read only access*
    
    This one does not support subobjects. 
    """
    implements(INonContainer, IObject, IReadonly)

class ObjectContainerBase(Object, ObjectEdit, ObjectWorkflow, Container, ContainerEdit,ContainerSecurity,  Events, ContainerFactory):
    """
    *Default container object with write access*
    
    This one supports subobjects. 
    """
    implements(IContainer, IObject)
    
class ObjectContainerReadOnlyBase(Object, ObjectWorkflow, Container, Events, ContainerFactory):
    """
    *Container object with read only access and cache*
    
    This one supports subobjects and caches them in memory. 
    """
    implements(IObject, IContainer, IReadonly)



# page elements for subclassing --------------------------------------------
from nive.components.extensions.path import AlternatePath
from nive.components.extensions.pages import PageContainer, PageElementContainer, PageElement, PageColumns
from nive.cms.cmsview.sort import Sort
from nive.cms.cmsview.cutcopy import ObjCopy, ContainerCopy

class PageBase(ContainerCopy, Sort, AlternatePath, PageColumns, PageContainer, PageElementContainer, ObjectContainerBase):
    """
    *Page with content element support*
    
    - stored in database
    - rendered as website page
    - handles sub pages
    - handles page columns
    - is an element container
    - supports paste of elements and pages
    - contained pages and elements are sortable 
    
    Interfaces: ``IPage, IPageContainer, IPageElementContainer, IContainer, IObject``
    """
    implements(IPage, IPageContainer, IPageElementContainer)


class PageRootBase(ContainerCopy, Sort, AlternatePath, PageColumns, PageContainer, PageElementContainer, RootBase):
    """
    *Root with content element support*
    
    - handles sub pages
    - handles page columns
    - rendered as website page
    - is an element container
    - supports paste of elements and pages
    - contained pages and elements are sortable 
    
    Interfaces: ``IPageContainer, IPageElementContainer, IContainer, IRoot``
    """
    implements(IPageContainer, IPageElementContainer)


class PageElementBase(ObjCopy, PageElement, ObjectBase):
    """
    *Page element* 
    
    - stored in database
    - does not store subobjects
    - stored in element containers 
    - cut, copy and paste support
    
    Interfaces: ``INonContainer, IObject, IPageElement``
    """
    implements(IPageElement)


class PageElementFileBase(ObjCopy, PageElement, ObjectBase):
    """
    *Page element with file download support*
    
    - stored in database
    - does not store subobjects
    - stored in element containers 
    - cut, copy and paste support
    - contained files can be downloaded

    Interfaces: ``INonContainer, IObject, IPageElement, IFile``
    """
    implements(IPageElement, IFile)


class PageElementContainerBase(Sort, ContainerCopy, PageElementContainer, ObjectContainerBase):
    """
    *Element container*
    
    - stored in database
    - handles page elements
    - supports paste of elements and pages
    - contained pages and elements are sortable 

    Interfaces: ``IContainer, IObject, IPageElement, IPageElementContainer``
    """
    implements(IPageElement, IPageElementContainer)


class FolderBase(ContainerCopy, PageElement, ObjectContainerBase):
    """
    *Resource container*
    
    - stored in database
    - handles files and resource objects
    - supports paste of elements
    """
    implements(IFolder)



from nive.definitions import ModuleConf

configuration = ModuleConf(
    id="useradmin-module",
    name="Meta package for useradmin components",
    modules=(
        "nive.userdb.useradmin.adminroot",
        "nive.userdb.useradmin.view",
        "nive.components.reform.reformed"
    ),
)




# -*- coding: utf-8 -*-

import time
import unittest

from nive.helper import FormatConfTestFailure

from nive.cms import app, box, column, file, image, link, media, menublock, news, note, page, root
from nive.cms import spacer, text, code




class TestConf(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_conf1(self):
        r=app.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf3(self):
        r=box.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf4(self):
        r=column.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf5(self):
        r=file.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf7(self):
        r=image.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf8(self):
        r=link.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf9(self):
        r=media.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf10(self):
        r=menublock.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf11(self):
        r=news.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf12(self):
        r=note.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf13(self):
        r=page.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf14(self):
        r=root.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf15(self):
        r=spacer.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf16(self):
        r=text.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf17(self):
        r=code.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")



if __name__ == '__main__':
    unittest.main()



#
""" -*- coding: utf-8 -*-"""

from copy import deepcopy


# default ------------------------------------------------
empty = {}
empty["code"] = u""
empty["code2"] = u""
empty["name"] = u""
empty["articles_used"] = 0 # 1=article, 2=f+m, 3=f+m+n
empty["articles"] = {}
empty["article_abbr"] = ""
empty["verb_prefix"] = ""
empty["plural"] = ""
empty["codepage"] = ""
empty["remove_chars"] = "\t\r\n"
empty["special_chars"] = u""


#[s] languages -------------------------------------------

noArticle = "keine Zuordnung"
languages = ("ger", "eng", "ita", "spa", "fra", "por", "lat", "dan", "dut", "pol", "swe", "cze", "tur", "epo",
             "rus", "fin", "gre", "grc", "hrv", "rum", "afr", "ara", "alb", "amh", "arm", "aze", "bel", "ben",
             "tib", "bos", "bul", "cat", "zho", "hrv", "esp", "est", "fil", "fin", "kat", "ell", "hau",
             "heb", "hin", "hun", "isi", "ind", "jpn", "kor", "lav", "lit", "mon", "nep", "nor", "per", "pol",
             "ron", "san", "srp", "slk", "swa", "tha", "ukr", "vie")

# ger ---------------------------------------------------
ger = deepcopy(empty)
ger["code"] = u"ger"
ger["code2"] = u"de"
ger["name"] = u"Deutsch"
ger["articles_used"] = 3
ger["articles"] = {"m": "der", "f": "die", "n": "das"}
ger["verb_prefix"] = ""
ger["plural"] = "s"
ger["codepage"] = "iso-8859-15"
ger["special_chars"] = u"äöüÄÖÜß"

# eng ---------------------------------------------------
eng = deepcopy(empty)
eng["code"] = u"eng"
eng["code2"] = u"en"
eng["name"] = u"English"
eng["articles_used"] = 1
eng["articles"] = {"a": "the"}
eng["verb_prefix"] = "to"
eng["plural"] = "s"
eng["codepage"] = "iso-8859-1"
eng["special_chars"] = u""

# ita ---------------------------------------------------
ita = deepcopy(empty)
ita["code"] = u"ita"
ita["code2"] = u"it"
ita["name"] = u"Italiano"
ita["articles_used"] = 2
ita["articles"] = {"m": "il", "f": "la"}
ita["article_abbr"] = "l'"
ita["verb_prefix"] = ""
ita["plural"] = ""
ita["codepage"] = "iso-8859-3"
ita["special_chars"] = u"ÀÈÉÌÒÙàèéìòù"

# spa ---------------------------------------------------
spa = deepcopy(empty)
spa["code"] = u"spa"
spa["code2"] = u"es"
spa["name"] = u"Español"
spa["articles_used"] = 2
spa["articles"] = {"m": "el", "f": "la"}
spa["article_abbr"] = ""
spa["verb_prefix"] = ""
spa["plural"] = "s"
spa["codepage"] = "iso-8859-3"
spa["special_chars"] = u"ÁÉÍÑÓÚÜáéíñóúü"


# fra ---------------------------------------------------
fra = deepcopy(empty)
fra["code"] = u"fra"
fra["code2"] = u"fr"
fra["name"] = u"Français"
fra["articles_used"] = 2
fra["articles"] = {"m": "le", "f": "la"}
fra["article_abbr"] = "l’"
fra["verb_prefix"] = ""
fra["plural"] = ""
fra["codepage"] = "iso-8859-3"
fra["special_chars"] = u"ÀÂÇÈÉÊËÎÏÔŒÙÛÜŸàâçèéêëîïôœùûüÿ"


# por ---------------------------------------------------
por = deepcopy(empty)
por["code"] = u"por"
por["code2"] = u"pt"
por["name"] = u" Português"
por["articles_used"] = 2
por["articles"] = {"m": "o", "f": "a"}
por["article_abbr"] = ""
por["verb_prefix"] = ""
por["plural"] = ""
por["codepage"] = "iso-8859-3"
por["special_chars"] = u"ÀÁÂÃÇÉÊÍÓÔÕÚÜàáâãçéêíóôõúü"


# lat ---------------------------------------------------
lat = deepcopy(empty)
lat["code"] = u"lat"
lat["code2"] = u"la"
lat["name"] = u"Lingua Latina"
lat["articles_used"] = 0
lat["articles"] = {}
lat["article_abbr"] = ""
lat["verb_prefix"] = ""
lat["plural"] = ""
lat["codepage"] = "iso-8859-3"
lat["special_chars"] = u"ÆŒæœ"

# dan ---------------------------------------------------
dan = deepcopy(empty)
dan["code"] = u"dan"
dan["code2"] = u"da"
dan["name"] = u"Dansk"
dan["articles_used"] = 0
dan["articles"] = {}
dan["article_abbr"] = ""
dan["verb_prefix"] = ""
dan["plural"] = "r, e"
dan["codepage"] = "iso-8859-10"
dan["special_chars"] = u"ÅÆØåæø"

# dut ---------------------------------------------------
dut = deepcopy(empty)
dut["code"] = u"dut"
dut["code2"] = u"nl"
dut["name"] = u"Nederlands"
dut["articles_used"] = 3
dut["articles"] = {"m": "de",  "f": "de", "n": "het"}
dut["article_abbr"] = ""
dut["verb_prefix"] = ""
dut["plural"] = ""
dut["codepage"] = "iso-8859-10"
dut["special_chars"] = u"ÄÈÉËÏĲÖÜäèéëïĳöü"

# pol ---------------------------------------------------
pol = deepcopy(empty)
pol["code"] = u"pol"
pol["code2"] = u"pl"
pol["name"] = u"Polski"
pol["articles_used"] = 4
pol["articles"] = {"mb": "maskulin belebt", "mu": "maskulin unbelebt", "f": "feminin", "n": "neutrum"}
pol["article_abbr"] = ""
pol["verb_prefix"] = ""
pol["plural"] = ""
pol["codepage"] = "iso-8859-2"
pol["special_chars"] = u"ĄĆĘŁŃÓŚŹŻąćęłłńóśźż "

# swe ---------------------------------------------------
swe = deepcopy(empty)
swe["code"] = u"swe"
swe["code2"] = u"sv"
swe["name"] = u"Svenska"
swe["articles_used"] = 2
swe["articles"] = {"u": "en", "n": "ett"}
swe["article_abbr"] = ""
swe["verb_prefix"] = ""
swe["plural"] = ""
swe["codepage"] = "iso-8859-10"
swe["special_chars"] = u"ÄÅÖäåö "

# cze ---------------------------------------------------
cze = deepcopy(empty)
cze["code"] = u"cze"
cze["code2"] = u"cs"
cze["name"] = u" Čeština"
cze["articles_used"] = 4
cze["articles"] = {"mb": "maskulin belebt", "mu": "maskulin unbelebt", "f": "feminin", "n": "neutrum"}
cze["article_abbr"] = ""
cze["verb_prefix"] = ""
cze["plural"] = ""
cze["codepage"] = "iso-8859-2"
cze["special_chars"] = u"ÁČĎÉĚÍŇÓŘŠŤÚŮÝŽáčďéĕíňóřšťúŭýž"

# tur ---------------------------------------------------
tur = deepcopy(empty)
tur["code"] = u"tur"
tur["code2"] = u"tr"
tur["name"] = u" Türkçe"
tur["articles_used"] = 0
tur["articles"] = { }
tur["article_abbr"] = ""
tur["verb_prefix"] = ""
tur["plural"] = ""
tur["codepage"] = "iso-8859-9"
tur["special_chars"] = u"ÂÇĞİÖŞÜâçğıöşü"

# epo ---------------------------------------------------
epo = deepcopy(empty)
epo["code"] = u"epo"
epo["code2"] = u"eo"
epo["name"] = u"Esperanto"
epo["articles_used"] = 0
epo["articles"] = { }
epo["article_abbr"] = ""
epo["verb_prefix"] = ""
epo["plural"] = ""
epo["codepage"] = "iso-8859-3"
epo["special_chars"] = u"ĈĜĤĴŜŬĉĝĥĵŝǔ"

# fin ---------------------------------------------------
fin = deepcopy(empty)
fin["code"] = u"fin"
fin["code2"] = u"fi"
fin["name"] = u"Suomi"
fin["articles_used"] = 0
fin["articles"] = { }
fin["article_abbr"] = ""
fin["verb_prefix"] = ""
fin["plural"] = ""
fin["codepage"] = "iso-8859-10"
fin["special_chars"] = u"ÄÅÖŠŽäåöšž"

# rus ---------------------------------------------------
rus = deepcopy(empty)
rus["code"] = u"rus"
rus["code2"] = u"ru"
rus["name"] = u"Ру́сский"
rus["articles_used"] = 4
rus["articles"] = {"mb": "maskulin belebt", "mu": "maskulin unbelebt", "f": "feminin", "n": "neutrum"}
rus["article_abbr"] = ""
rus["verb_prefix"] = ""
rus["plural"] = ""
rus["codepage"] = "iso-8859-5"
rus["special_chars"] = u"АаБбВвЃѓДдЂђЕеЁёЖжЗзИиЙйКкЛлМмНнОоПпРрСсТтЌќЋћУуЎўФфЦцЧчШшЬьѢѣ"

# gre ---------------------------------------------------
gre = deepcopy(empty)
gre["code"] = u"gre"
gre["code2"] = u"el"
gre["name"] = u"Ελληνικα"
gre["articles_used"] = 3
gre["articles"] = {"m": "o", "f": "η", "n": "το"}
gre["article_abbr"] = ""
gre["verb_prefix"] = ""
gre["plural"] = ""
gre["codepage"] = "iso-8859-7"
gre["special_chars"] = u"ΑΒΓΔΕΖΗΘΙΚΛΜΝΞΟΠΡΣΤΥΦΧΨΩαβγδεζηθικλμνξοπρστυφχψως"

# grc ---------------------------------------------------
grc = deepcopy(empty)
grc["code"] = u"grc"
grc["code2"] = u"gr"
grc["name"] = u"ἡ Ἑλληνικὴ γλῶσσα"
grc["articles_used"] = 3
grc["articles"] = {"m": "o", "f": "η", "n": "το"}
grc["article_abbr"] = ""
grc["verb_prefix"] = ""
grc["plural"] = ""
grc["codepage"] = "iso-8859-7"
grc["special_chars"] = u"ΑΒΓΔΕΖΗΘΙΚΛΜΝΞΟΠΡΣΤΥΦΧΨΩαβγδεζηθικλμνξοπρστυφχψως"

# hrv ---------------------------------------------------
hrv = deepcopy(empty)
hrv["code"] = u"hrv"
hrv["code2"] = u"hr"
hrv["name"] = u" Hrvatski"
hrv["articles_used"] = 0
hrv["articles"] = {}
hrv["article_abbr"] = ""
hrv["verb_prefix"] = ""
hrv["plural"] = ""
hrv["codepage"] = "iso-8859-2"
hrv["special_chars"] = u"ĆČĐŠŽćčđšž"

# rum ---------------------------------------------------
rum = deepcopy(empty)
rum["code"] = u"rum"
rum["code2"] = u"ro"
rum["name"] = u"Română"
rum["articles_used"] = 3
rum["articles"] = {"m": "m", "f": "f"}
rum["article_abbr"] = "l'"
rum["verb_prefix"] = ""
rum["plural"] = ""
rum["codepage"] = "iso-8859-3"
rum["special_chars"] = u"ÂĂÎȘȚâăîșț"

# afr ---------------------------------------------------
afr = deepcopy(empty)
afr["code"] = u"afr"
afr["code2"] = u"af"
afr["name"] = u"Afrikaans"
afr["codepage"] = "iso-8859-10"
afr["special_chars"] = u"ÈèÉéÊêËëÎîÏïÔôÛû"

# ara ---------------------------------------------------
ara = deepcopy(empty)
ara["code"] = u"ara"
ara["code2"] = u"ar"
ara["name"] = u"للغة العربية‎"
ara["codepage"] = "iso-8859-6"
ara["special_chars"] = u""

# alb ---------------------------------------------------
alb = deepcopy(empty)
alb["code"] = u"alb"
alb["code2"] = u"sq"
alb["name"] = u"Gjuha Shqipe"
alb["codepage"] = "iso-8859-3"
alb["special_chars"] = u"ËëÇç"

# amh ---------------------------------------------------
amh = deepcopy(empty)
amh["code"] = u"amh"
amh["code2"] = u"am"
amh["name"] = u"Amarəñña"
amh["codepage"] = ""
amh["special_chars"] = u""

# arm -------------------------------------------------
arm = deepcopy(empty)
arm["code"] = u"arm"
arm["code2"] = u"hy"
arm["name"] = u"Hajeren lesu"
arm["codepage"] = ""
arm["special_chars"] = u""

# aze ---------------------------------------------------
aze = deepcopy(empty)
aze["code"] = u" aze "
aze["code2"] = u"tr"
aze["name"] = u"Azərbaycan dili"
aze["codepage"] = "iso-8859-9"
aze["special_chars"] = u"ƏəÂÇĞİÖŞÜâçğıöşü"

# bel ---------------------------------------------------
bel = deepcopy(empty)
bel["code"] = u"bel"
bel["code2"] = u"be"
bel["name"] = u"беларуская мова"
bel["articles_used"] = 4
bel["articles"] = {"mb": "maskulin belebt", "mu": "maskulin unbelebt", "f": "feminin", "n": "neutrum"}
bel["article_abbr"] = ""
bel["verb_prefix"] = ""
bel["plural"] = ""
bel["codepage"] = "iso-8859-5"
bel["special_chars"] = u"АаБбВвЃѓДдЂђЕеЁёЖжЗзИиЙйКкЛлМмНнОоПпРрСсТтЌќЋћУуЎўФфЦцЧчШшЬьѢѣ"

# ben---------------------------------------------------
ben = deepcopy(empty)
ben["code"] = u"ben"
ben["code2"] = u"bn"
ben["name"] = u"Bangla bhasha"
ben["codepage"] = ""
ben["special_chars"] = u""

# tib------------------------------------------------
tib = deepcopy(empty)
tib["code"] = u" tib "
tib["code2"] = u"bo"
tib["name"] = u"Bod yig"
tib["codepage"] = ""
tib["special_chars"] = u""

# bos---------------------------------------------------
bos = deepcopy(empty)
bos["code"] = u"bos"
bos["code2"] = u"bs"
bos["name"] = u"Bosanski jezik"
bos["codepage"] = "iso-8859-2"
bos["special_chars"] = u"čćžšž"

# bul ---------------------------------------------------
bul = deepcopy(empty)
bul["code"] = u"bul"
bul["code2"] = u"bg"
bul["name"] = u"Български език"
bul["articles_used"] = 4
bul["articles"] = {"mb": "maskulin belebt", "mu": "maskulin unbelebt", "f": "feminin", "n": "neutrum"}
bul["article_abbr"] = ""
bul["verb_prefix"] = ""
bul["plural"] = ""
bul["codepage"] = "iso-8859-5"
bul["special_chars"] = u"АаБбВвЃѓДдЂђЕеЁёЖжЗзИиЙйКкЛлМмНнОоПпРрСсТтЌќЋћУуЎўФфЦцЧчШшЬьѢѣ"

# cat ---------------------------------------------------
cat = deepcopy(empty)
cat["code"] = u"cat"
cat["code2"] = u"ca"
cat["name"] = u"Catalán"
cat["articles_used"] = 2
cat["articles"] = {"m": "el", "f": "la"}
cat["article_abbr"] = ""
cat["verb_prefix"] = ""
cat["plural"] = "s"
cat["codepage"] = "iso-8859-3"
cat["special_chars"] = u"ÁÉÍÑÓÚÜáéíñóúü"

# zho ------------------------------------------------
zho = deepcopy(empty)
zho["code"] = u"zho"
zho["code2"] = u"zh"
zho["name"] = u"中文"
zho["codepage"] = ""
zho["special_chars"] = u""

# hrv ---------------------------------------------------
hrv = deepcopy(empty)
hrv["code"] = u"hrv"
hrv["code2"] = u"sh"
hrv["name"] = u"Hrvatski jezik"
hrv["codepage"] = "iso-8859-2"
hrv["special_chars"] = u"čćžšž"

# esp ---------------------------------------------------
esp = deepcopy(empty)
esp["code"] = u"esp"
esp["code2"] = u"ep"
esp["name"] = u"Esperanto"
esp["codepage"] = "iso-8859-3"

# est ---------------------------------------------------
est = deepcopy(empty)
est["code"] = u"est"
est["code2"] = u"et"
est["name"] = u"Eesti keel"
est["codepage"] = "iso-8859-1"

# fil ---------------------------------------------------
fil = deepcopy(empty)
fil["code"] = u"fil"
fil["code2"] = u"fl"
fil["name"] = u"Filipino"
fil["codepage"] = "iso-8859-1"

# fin ---------------------------------------------------
fin = deepcopy(empty)
fin["code"] = u"fin"
fin["code2"] = u"fi"
fin["name"] = u"Suomi"
fin["codepage"] = "iso-8859-1"
fin["special_chars"] = u"öÖäÄåÅ"

# kat ---------------------------------------------------
kat = deepcopy(empty)
kat["code"] = u"kat"
kat["code2"] = u"ka"
kat["name"] = u"Kartuli Ena"
kat["codepage"] = ""

# ell ---------------------------------------------------
ell = deepcopy(empty)
ell["code"] = u"ell"
ell["code2"] = u"el"
ell["name"] = u"ελληνική γλώσσα"
ell["codepage"] = "iso-8859-7"

# hau ---------------------------------------------------
hau = deepcopy(empty)
hau["code"] = u"hau"
hau["code2"] = u"ha"
hau["name"] = u"Hausa"
hau["codepage"] = "iso-8859-1"

# heb ---------------------------------------------------
heb = deepcopy(empty)
heb["code"] = u"heb"
heb["code2"] = u"iw"
heb["name"] = u" עברית"
heb["codepage"] = "iso-8859-8"

# hin ---------------------------------------------------
hin = deepcopy(empty)
hin["code"] = u"hin"
hin["code2"] = u"hi"
hin["name"] = u"Hindi"
hin["codepage"] = ""

# hun ---------------------------------------------------
hun = deepcopy(empty)
hun["code"] = u"hun"
hun["code2"] = u"hu"
hun["name"] = u"Magyar nyelv"
hun["codepage"] = "iso-8859-1"

# isi ---------------------------------------------------
isi = deepcopy(empty)
isi["code"] = u"isi"
isi["code2"] = u"is"
isi["name"] = u"Íslenska"
isi["codepage"] = "iso-8859-1"

# ind ---------------------------------------------------
ind = deepcopy(empty)
ind["code"] = u"ind"
ind["code2"] = u"in"
ind["name"] = u"Bahasa Indonesia"
ind["codepage"] = "iso-8859-1"

# jpn ---------------------------------------------------
jpn = deepcopy(empty)
jpn["code"] = u"jpn"
jpn["code2"] = u"ja"
jpn["name"] = u"日本語"
jpn["codepage"] = ""

# kor ---------------------------------------------------
kor = deepcopy(empty)
kor["code"] = u"kor"
kor["code2"] = u"ko"
kor["name"] = u"韓國語"
kor["codepage"] = ""

# lav ---------------------------------------------------
lav = deepcopy(empty)
lav["code"] = u"lav"
lav["code2"] = u"lv"
lav["name"] = u"Latviešu valoda"
lav["codepage"] = "iso-8859-4"

# lit ---------------------------------------------------
lit = deepcopy(empty)
lit["code"] = u"lit"
lit["code2"] = u"lt"
lit["name"] = u"Lietuvių kalba"
lit["codepage"] = "iso-8859-4"

# mon ---------------------------------------------------
mon = deepcopy(empty)
mon["code"] = u"mon"
mon["code2"] = u"mn"
mon["name"] = u"халх монгол хэл"
mon["codepage"] = "iso-8859-5"

# nep ---------------------------------------------------
nep = deepcopy(empty)
nep["code"] = u"nep"
nep["code2"] = u"ne"
nep["name"] = u"Nepali"
nep["codepage"] = ""

# nor ---------------------------------------------------
nor = deepcopy(empty)
nor["code"] = u"nor"
nor["code2"] = u"no"
nor["name"] = u"Norsk"
nor["codepage"] = "iso-8859-1"

# per ---------------------------------------------------
per = deepcopy(empty)
per["code"] = u"per"
per["code2"] = u"fa"
per["name"] = u"فارسی"
per["codepage"] = "iso-8859-6"

# pol ---------------------------------------------------
pol = deepcopy(empty)
pol["code"] = u"pol"
pol["code2"] = u"pl"
pol["name"] = u"Język polski"
pol["codepage"] = "iso-8859-2"

# ron ---------------------------------------------------
ron = deepcopy(empty)
ron["code"] = u"ron"
ron["code2"] = u"ro"
ron["name"] = u"Limba româna"
ron["codepage"] = "iso-8859-1"

# san ---------------------------------------------------
san = deepcopy(empty)
san["code"] = u"san"
san["code2"] = u"sa"
san["name"] = u"Sanskrit"
san["codepage"] = ""

# srp ---------------------------------------------------
srp = deepcopy(empty)
srp["code"] = u"srp"
srp["code2"] = u"sr"
srp["name"] = u"Srpski jezik"
srp["codepage"] = "iso-8859-2"

# slk ---------------------------------------------------
slk = deepcopy(empty)
slk["code"] = u"slk"
slk["code2"] = u"sk"
slk["name"] = u"slovenčina"
slk["codepage"] = "iso-8859-2"

# swa ---------------------------------------------------
swa = deepcopy(empty)
swa["code"] = u"swa"
swa["code2"] = u"sw"
swa["name"] = u"Kiswahili"
swa["codepage"] = "iso-8859-1"

# tha ---------------------------------------------------
tha = deepcopy(empty)
tha["code"] = u"tha"
tha["code2"] = u"th"
tha["name"] = u"ภาษาไทย"
tha["codepage"] = "iso-8859-11"

# ukr ---------------------------------------------------
ukr = deepcopy(empty)
ukr["code"] = u"ukr"
ukr["code2"] = u"uk"
ukr["name"] = u"українська"
ukr["codepage"] = "iso-8859-5"

# vie ---------------------------------------------------
vie = deepcopy(empty)
vie["code"] = u"vie"
vie["code2"] = u"vi"
vie["name"] = u"Tiếng Việt"
vie["codepage"] = "iso-8859-1"



#[s] functions -------------------------------------------

global _cl
_cl = []
for l in languages:
    lang = globals().get(l)
    _cl.append({"id": lang["code"], "name": lang["name"]})

def GetConf(langcode):
    """
    Load language configuration by code (3 letters)
    """
    if langcode in languages:
        return globals().get(langcode)
    if len(langcode)==2:
        for l in languages:
            c = globals().get(l)
            if c["code2"] == langcode:
                return c
    return empty


def GetLanguages():
    """
    load codelist of languages and cache
    """
    return _cl
    #cl = []
    #for l in languages:
    #    cl.append({"id": l["code"], "name": l["name"]})
    #_cl = cl
    #return cl
